/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "elementbrowser.h"
#include "konduititemdelegate.h"
#include "elementmodel.h"
#include "konduitshell.h"
#include "../src/element.h"

#include <KCategoryDrawer>
#include <KCategorizedSortFilterProxyModel>
#include <KDialog>
#include <KDebug>
#include <KSycoca>
#include <KStandardAction>
#include <KAction>

#include <QtGui/QHeaderView>
#include <QtGui/QVBoxLayout>
#include <QtCore/QEvent>
#include <QTimer>
#include <QAction>
#include <QMessageBox>
#include <QMenu>

#include <Plasma/Applet>


class ElementBrowser::Private
{
public:
    QTimer *timer;
    QString name;
    QMenu *bbMenu;
    QString activePluginName;
};


ElementBrowser::ElementBrowser( QWidget* parent )
    : KCategorizedView( parent ), d( new Private )
{
    //QTimer::singleShot(0, this, SLOT( init() ));
    init();
    setupActions();
}

void ElementBrowser::init()
{
    setCategoryDrawer(new KCategoryDrawer());
    setViewMode(QListView::IconMode);
    setItemDelegate( new KonduitItemDelegate( this ) );
    setDragDropMode( QAbstractItemView::DragOnly );
    setSpacing( KDialog::spacingHint() );
    
    m_model = new ElementModel( this );
    m_model->populate();
    
    m_proxyModel = new KCategorizedSortFilterProxyModel( this );
    m_proxyModel->setCategorizedModel(true);
    m_proxyModel->setSourceModel(m_model);
    setModel( m_proxyModel );
    
}


void ElementBrowser::setupActions()
{
    m_sycoca = new KSycoca();
    //WARNING: databaseChanged(QStringList) only exists since KDE 4.3
    connect( m_sycoca, SIGNAL(databaseChanged (QStringList)), 
             this, SLOT(triggerPluginUpdate(QStringList)) );
    
    QAction *openBlackBoxAction = KStandardAction::open( this, SLOT( slotOpenBlackBox() ), this );
    openBlackBoxAction->setText( tr("Open black box") );
    openBlackBoxAction->setShortcut( 0 );
    
    QAction *deleteBlackBoxAction = KStandardAction::cut( this, SLOT( slotDeleteBlackBox() ), this );
    deleteBlackBoxAction->setText( tr("Delete black box") );
    deleteBlackBoxAction->setShortcut( 0 );
    
    QAction *exportBlackBoxAction = KStandardAction::save( this, SLOT( slotExportBlackBox() ), this );
    exportBlackBoxAction->setText( tr("Export black box") );
    exportBlackBoxAction->setStatusTip(tr("Exports the black box to a file"));
    exportBlackBoxAction->setShortcut( 0 );
    
    d->bbMenu = new QMenu(this);
    d->bbMenu->addAction( openBlackBoxAction );
    d->bbMenu->addAction( deleteBlackBoxAction );
    d->bbMenu->addAction( exportBlackBoxAction );
}


ElementBrowser::~ElementBrowser()
{
}


void ElementBrowser::triggerPluginUpdate( QStringList resources )
{
    //delay updating, otherwise it's done too quickly
    if(resources.contains("services")) {
        QTimer::singleShot(1000, this, SLOT(refresh()));
    }    
}


void ElementBrowser::refresh()
{
    kDebug() << "Refreshing";
    // m_model->populate();
    // setModel( m_proxyModel );
    //FIXME: try to refresh with less operations
    init();
}


void ElementBrowser::startDrag( Qt::DropActions supportedActions )
{
    // Work around KCategorizedView weirdness
    QListView::startDrag( supportedActions );
}


void ElementBrowser::slotOpenBlackBox( )
{
    KonduitShell *ksh = (KonduitShell*) parentWidget()->parentWidget()->parentWidget();
    ksh->slotOpenBlackBox(d->activePluginName);
}

void ElementBrowser::slotDeleteBlackBox( )
{
    KonduitShell *ksh = (KonduitShell*) parentWidget()->parentWidget()->parentWidget();
    ksh->slotDeleteBlackBox(d->activePluginName);
}

void ElementBrowser::slotExportBlackBox( )
{
    KonduitShell *ksh = (KonduitShell*) parentWidget()->parentWidget()->parentWidget();
    ksh->slotExportBlackBox(d->activePluginName);
}

void ElementBrowser::contextMenuEvent ( QContextMenuEvent * event )
{
    QVariant qv = m_model->data( indexAt( event->pos() ), ElementModel::PluginInfoRole );
    if( qv.isNull() ) { //if not a plugin
        return;
    }
    
    KPluginInfo kpi = qv.value<KPluginInfo>();
    
    d->activePluginName = kpi.name();
    
    //FIXME: use type reflection here. Something like dynamic_cast<Konduit::Elements::BlackboxLoader*>(this)
    //popup context menu for blackboxes
    if( !kpi.pluginName().startsWith("konduit_" )) {//only add this if it is a black box
        d->bbMenu->popup( event->globalPos() );
    }
}


#include "elementbrowser.moc"
