/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "elementmodel.h"

#include <QtCore/QMimeData>

#include <plasma/applet.h>
#include <KIcon>
#include <KDebug>
#include "kcategorizedsortfilterproxymodel.h"

#include "../src/element.h"

#undef DOLPHIN_DRAGANDDROP

ElementModel::ElementModel( QObject* parent )
    : QStringListModel( parent )
{
}


ElementModel::~ElementModel()
{
}


void ElementModel::populate()
{
    removeRows( 0, rowCount() );
    m_plugins.clear();
    
    foreach ( const QString& category, Konduit::Element::listAppletCategories() ) {
        foreach( const KPluginInfo& kpi, Plasma::Applet::listAppletInfo(category) ) {
            m_plugins.append(kpi);
        }
    }
    
    insertRows(0, m_plugins.count());//have a corresponding row for each plugin
    kDebug() << "Found" << m_plugins.count() << "Konduit plugins";

    reset();
}


Qt::ItemFlags ElementModel::flags( const QModelIndex& index ) const
{
    if ( index.isValid() ) {
        return Qt::ItemIsDragEnabled|Qt::ItemIsSelectable|Qt::ItemIsEnabled;
    }
    else {
        return QAbstractItemModel::flags( index );
    }
}


QVariant ElementModel::data( const QModelIndex& index, int role ) const
{
    if ( index.isValid() && index.row() < m_plugins.count() ) {
        switch( role ) {
        case Qt::DisplayRole:
            return m_plugins[index.row()].name();

        case Qt::DecorationRole:
            return KIcon( m_plugins[index.row()].icon() );

        case Qt::ToolTipRole:
            return m_plugins[index.row()].comment();

        case PluginInfoRole:
            return qVariantFromValue<KPluginInfo>( m_plugins[index.row()] );

        case KCategorizedSortFilterProxyModel::CategorySortRole:
        case KCategorizedSortFilterProxyModel::CategoryDisplayRole: {
            QString category = m_plugins[index.row()].category();
            category.replace("Konduit Elements - ", "", Qt::CaseSensitive);
            //kDebug() << "returning CategoryDisplayRole: " << category;
            return category;
            }
            break;
        default:
            return QVariant();
            break;
        }
    }
    else {
        return QVariant();
    }

}


QStringList ElementModel::mimeTypes() const
{
    return QStringList() << QLatin1String("text/x-konduitelementname");
}


QMimeData* ElementModel::mimeData( const QModelIndexList& indexes ) const
{
    QMimeData* mimeData = new QMimeData();
    QByteArray encodedData;

    QStringList names;

    foreach ( QModelIndex index, indexes ) {
        if ( index.isValid() ) {
            // encode the plugin name
            names << m_plugins[index.row()].pluginName();
        }
    }

    QDataStream stream( &encodedData, QIODevice::WriteOnly );
    stream << names;

    mimeData->setData( mimeTypes()[0], encodedData );
    return mimeData;
}


#include "elementmodel.moc"
