/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * Painting code based on kcategorizeditemsviewdelegate.cpp
 * Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "konduititemdelegate.h"
#include "elementmodel.h"

#include <QtGui/QPainter>
#include <QtGui/QFontMetrics>

#include <kplugininfo.h>
#include <KIcon>


#define UNIVERSAL_PADDING 6
#define FADE_LENGTH 32
#define MAIN_ICON_SIZE 48


KonduitItemDelegate::KonduitItemDelegate( QObject* parent )
    : QAbstractItemDelegate( parent )
{
}


void KonduitItemDelegate::paint( QPainter* painter,
                                 const QStyleOptionViewItem& option,
                                 const QModelIndex& index ) const
{
    // Preparing needed data for painting
    int left = option.rect.left();
    int top = option.rect.top();
    int width = option.rect.width();
//    int height = option.rect.height();
    bool leftToRight = (painter->layoutDirection() == Qt::LeftToRight);

    QIcon::Mode iconMode = QIcon::Normal;

    QColor backgroundColor = (option.state.testFlag(QStyle::State_Selected))?
            option.palette.color(QPalette::Highlight):option.palette.color(QPalette::Base);
    QColor foregroundColor = (option.state.testFlag(QStyle::State_Selected))?
            option.palette.color(QPalette::HighlightedText):option.palette.color(QPalette::Text);

    // Base Background
    painter->fillRect(option.rect, QBrush(backgroundColor));

    // Painting main column
    QStyleOptionViewItem local_option_title(option);
    QStyleOptionViewItem local_option_normal(option);

    local_option_title.font.setBold(true);
    local_option_title.font.setPointSize(local_option_title.font.pointSize() + 2);

    QLinearGradient gradient;

    KPluginInfo plugin = index.data( ElementModel::PluginInfoRole ).value<KPluginInfo>();

    QString title = plugin.name();
    QString description = plugin.comment();

    // Painting

    // Text
    int textInner = 2 * UNIVERSAL_PADDING + MAIN_ICON_SIZE;

    painter->setPen(foregroundColor);
    painter->setFont(local_option_title.font);
    painter->drawText(
        left + (leftToRight ? textInner : 0),
        top + UNIVERSAL_PADDING,
        width - textInner, MAIN_ICON_SIZE / 2,
        Qt::AlignBottom | Qt::AlignLeft, title);
    painter->setFont(local_option_normal.font);
    painter->drawText(
        left + (leftToRight ? textInner : 0),
        top + UNIVERSAL_PADDING + MAIN_ICON_SIZE / 2,
        width - textInner, MAIN_ICON_SIZE / 2,
        Qt::AlignTop | Qt::AlignLeft, description);

    // Main icon
    KIcon( plugin.icon() ).paint(painter,
                                 leftToRight ? left + UNIVERSAL_PADDING : left + width - UNIVERSAL_PADDING - MAIN_ICON_SIZE,
                                 top + UNIVERSAL_PADDING,
                                 MAIN_ICON_SIZE, MAIN_ICON_SIZE, Qt::AlignCenter, iconMode);

    // Gradient part of the background - fading of the text at the end
    if (leftToRight) {
        gradient = QLinearGradient(left + width - UNIVERSAL_PADDING - FADE_LENGTH, 0,
                                   left + width - UNIVERSAL_PADDING, 0);
        gradient.setColorAt(1, backgroundColor);
        backgroundColor.setAlpha(0);
        gradient.setColorAt(0, backgroundColor);
    } else {
        gradient = QLinearGradient(left + UNIVERSAL_PADDING, 0,
                                   left + UNIVERSAL_PADDING + FADE_LENGTH, 0);
        gradient.setColorAt(0, backgroundColor);
        backgroundColor.setAlpha(0);
        gradient.setColorAt(1, backgroundColor);

    }

    QRect paintRect = option.rect;
    painter->setCompositionMode(QPainter::CompositionMode_SourceOver);
    painter->fillRect(paintRect, gradient);

    if (leftToRight) {
        gradient.setStart(left + width - FADE_LENGTH, 0);
        gradient.setFinalStop(left + width, 0);
    } else {
        gradient.setStart(left + UNIVERSAL_PADDING, 0);
        gradient.setFinalStop(left + UNIVERSAL_PADDING + FADE_LENGTH, 0);
    }
    paintRect.setHeight(UNIVERSAL_PADDING + MAIN_ICON_SIZE / 2);
    painter->fillRect(paintRect, gradient);
}


QSize KonduitItemDelegate::sizeHint( const QStyleOptionViewItem& option,
                                     const QModelIndex& index ) const
{
    if ( index.isValid() ) {
        KPluginInfo plugin = index.data( ElementModel::PluginInfoRole ).value<KPluginInfo>();
        QFont font( option.font );
        int width = QFontMetrics( font ).width( plugin.comment() ) + 2 * UNIVERSAL_PADDING;
        font.setBold( true );
        font.setPointSize( font.pointSize() + 2 );
        width = qMax( QFontMetrics( font ).width( plugin.name() ) + 2 * UNIVERSAL_PADDING, width );
        return QSize( width, MAIN_ICON_SIZE + 2 * UNIVERSAL_PADDING );
    }
    else {
        return QSize();
    }
}

#include "konduititemdelegate.moc"
