/*
 * konduitshell.cpp
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 */
#include "konduitshell.h"
#include "konduitshellview.h"
#include "settings.h"
#include "konduitscene.h"
#include "konduitselectiondialog.h"

#include "../src/konduitstoragemodel.h"
#include "../src/element.h"

#include <QtGui/QDropEvent>
#include <QtGui/QPainter>
#include <QtGui/QPrinter>
#include <QtGui/QGraphicsScene>

#include <kconfigdialog.h>
#include <kstatusbar.h>
#include <kaction.h>
#include <kactioncollection.h>
#include <kstandardaction.h>
#include <kstandardshortcut.h>
#include <kinputdialog.h>
#include <KDE/KLocale>
#include <KMessageBox>
#include <KFileDialog>
#include <KDebug>

#include <plasma/applet.h>


KonduitShell::KonduitShell()
    : KXmlGuiWindow(),
      m_view(new KonduitShellView(this)),
      m_printer(0)
{
    //setXMLFile("konduitshellui.rc");

    // accept dnd
    setAcceptDrops(true);

    // tell the KXmlGuiWindow that this is indeed the main widget
    setCentralWidget(m_view);

    // then, setup our actions
    setupActions();

    // add a status bar
    statusBar()->show();

    // a call to KXmlGuiWindow::setupGUI() populates the GUI
    // with actions, using KXMLGUI.
    // It also applies the saved mainwindow settings, if any, and ask the
    // mainwindow to automatically save settings if changed: window size,
    // toolbar position, icon size, etc.
    setupGUI();
    
    m_ksm = Konduit::Element::globalStorageModel();
}

KonduitShell::~KonduitShell()
{
}

void KonduitShell::setupActions()
{
    KStandardAction::openNew(this, SLOT(openNew()), actionCollection());
    KStandardAction::open(this, SLOT(fileOpen()), actionCollection());
    KStandardAction::quit(qApp, SLOT(quit()), actionCollection());
    KStandardAction::preferences(this, SLOT(optionsPreferences()), actionCollection());

    KAction* m_openBBAction = new KAction(this);
    m_openBBAction->setText("Open Black box");
    m_openBBAction->setIcon(KIcon("document-open"));
    m_openBBAction->setShortcut(Qt::CTRL + Qt::Key_B);
    m_openBBAction->setStatusTip(tr("Opens a Black box"));
    actionCollection()->addAction("openblackbox", m_openBBAction);
    connect(m_openBBAction, SIGNAL(triggered()), this, SLOT(fileOpenBlackBox()));

    KAction* m_insertAction = new KAction(this);
    m_insertAction->setText("Insert Konduit");
    m_insertAction->setIcon(KIcon("document-open"));
    m_insertAction->setShortcut(Qt::CTRL + Qt::Key_I);
    m_insertAction->setStatusTip(tr("Inserts a konduit project"));
    actionCollection()->addAction("insertkonduit", m_insertAction);
    connect(m_insertAction, SIGNAL(triggered()), this, SLOT(fileInsert()));
    
    KAction* m_importAction = new KAction(this);
    m_importAction->setText("Import Black box");
    m_importAction->setIcon(KIcon("document-open"));
    m_importAction->setShortcut(Qt::CTRL + Qt::Key_M);
    m_importAction->setStatusTip(tr("Imports a Black box from a file to the local repository"));
    actionCollection()->addAction("importblackbox", m_importAction);
    connect(m_importAction, SIGNAL(triggered()), this, SLOT(slotImportBlackBox()));
}

void KonduitShell::openNew()
{
    KonduitShell *ks;
    ks = new KonduitShell();
    ks->show();
}

void KonduitShell::fileOpen()
{
    QString name = KonduitSelectionDialog::selectKonduit( this, false);
    if ( !name.isEmpty() ) {
        if ( Konduit::Element* e = m_ksm->loadKonduit( name ) ) {

            KonduitShell* ks = new KonduitShell();

            QList<Konduit::Element*> el = e->allConnectedElements();
            foreach( Konduit::Element* elem, el ) {
                ks->m_view->scene()->addElement( elem );
            }
            ks->show();
        }
        else {
            KMessageBox::sorry( this, i18n( "Could not find saved konduit '%1'.", name ) );
        }
    }
}


void KonduitShell::fileInsert()
{
    QString name = KonduitSelectionDialog::selectKonduit( this, false);
    if ( !name.isEmpty() ) {
        if ( Konduit::Element* e = m_ksm->loadKonduit( name ) ) {
            QList<Konduit::Element*> el = e->allConnectedElements();
            foreach( Konduit::Element* elem, el ) {
                m_view->scene()->addElement( elem );
            }
        }
        else {
            KMessageBox::sorry( this, i18n( "Could not find saved konduit '%1'.", name ) );
        }
    }
}


void KonduitShell::fileOpenBlackBox()
{
    QString name = KonduitSelectionDialog::selectKonduit( this, true );
    if ( !name.isEmpty() ) {
        if ( Konduit::Element* e = m_ksm->loadKonduitBlackBox( name ) ) {

            KonduitShell* ks = new KonduitShell();

            QList<Konduit::Element*> el = e->allConnectedElements();
            foreach( Konduit::Element* elem, el ) {
                ks->m_view->scene()->addElement( elem );
            }

            ks->show();
        }
        else {
            KMessageBox::sorry( this, i18n( "Could not find saved konduit black box '%1'.", name ) );
        }
    }
}

void KonduitShell::slotOpenBlackBox( QString name )
{
    if ( Konduit::Element* e = m_ksm->loadKonduitBlackBox( name ) ) {

        KonduitShell* ks = new KonduitShell();

        QList<Konduit::Element*> el = e->allConnectedElements();
        foreach( Konduit::Element* elem, el ) {
            ks->m_view->scene()->addElement( elem );
        }

        ks->show();
    }
    else {
        KMessageBox::sorry( this, i18n( "Could not find saved konduit black box '%1'.", name ) );
    }
}

void KonduitShell::slotDeleteBlackBox( QString name )
{
    if ( m_ksm->deleteBlackBox( name ) ) {
        KMessageBox::information( this, i18n( "Deleted blackbox '%1'.", name ) );
    }
    else {
        KMessageBox::sorry( this, i18n( "Could not find blackbox '%1'.", name ) );
    }
}

void KonduitShell::slotExportBlackBox( QString name )
{
    QString fileName = KFileDialog::getSaveFileName(KUrl(), QString("*.kbb|Konduit Black box files"));
    
    if( !fileName.isEmpty() ) {
    
        QFile file( fileName );
        if( file.exists() ) { //confirm overwrite
            if ( KMessageBox::warningYesNo( 0, i18n( "Should the file '%1' be overwritten?", fileName ) ) == KMessageBox::No ) {
                return;
            }
        }
        
        if ( m_ksm->exportBlackBox( name, fileName ) ) {
            KMessageBox::information( this, i18n( "Successfully exported blackbox '%1' to %2", name, fileName ) );
        }
        else {
            KMessageBox::sorry( this, i18n( "Error exporting blackbox '%1' to %2!", name, fileName ) );
        }
   }
}

void KonduitShell::slotImportBlackBox()
{
    QString fileName = KFileDialog::getOpenFileName(KUrl(), QString("*.kbb|Konduit Black box files"));
    
    if( !fileName.isEmpty() ) {
        
        QString bbName = m_ksm->findBlackboxFromFile( fileName );
        if( !bbName.isEmpty() ) { //confirm overwrite
            if ( KMessageBox::warningYesNo( 0, i18n( "Should the blackbox '%1' in the repository be overwritten?", bbName ) ) == KMessageBox::No ) {
                return;
            }
        }
        
        if ( m_ksm->importBlackBox( fileName ) ) {
            KMessageBox::information( this, i18n( "Imported black box from '%1'" ).arg( fileName ) );
        }
        else {
            KMessageBox::sorry( this, i18n( "Error importing black box from  '%1'!" ).arg( fileName ) );
        }
    }
}


void KonduitShell::optionsPreferences()
{
    // The preference dialog is derived from prefs_base.ui
    //
    // compare the names of the widgets in the .ui file
    // to the names of the variables in the .kcfg file
    //avoid to have 2 dialogs shown
    if ( KConfigDialog::showDialog( "settings" ) )  {
        return;
    }
    KConfigDialog *dialog = new KConfigDialog(this, "settings", Settings::self());
    QWidget *generalSettingsDlg = new QWidget;
    ui_prefs_base.setupUi(generalSettingsDlg);
    dialog->addPage(generalSettingsDlg, i18n("General"), "package_setting");
    connect(dialog, SIGNAL(settingsChanged(QString)), m_view, SLOT(settingsChanged()));
    dialog->setAttribute( Qt::WA_DeleteOnClose );
    dialog->show();
}

#include "konduitshell.moc"
