/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "blackboxloaderelement.h"
#include "socket.h"
#include "plug.h"
#include "konduitstoragemodel.h"
#include "inputelement.h"
#include "outputelement.h"
#include "konduit_export.h"

#include <Soprano/Statement>
#include <Soprano/StatementIterator>

#include <QtCore/QQueue>
#include <QtCore/QDebug>
#include <QtGui/QPen>
#include <QtGui/QPainter>
#include <QGraphicsLinearLayout>
#include <QPushButton>
#include <QTimer>

#include <plasma/widgets/lineedit.h>
#include <plasma/widgets/pushbutton.h>

#include <KMessageBox>
#include <KPushButton>
#include <KDebug>
#include <KServiceTypeTrader>

class Konduit::Elements::BlackboxLoaderElement::Private
{
public:
    Private( BlackboxLoaderElement* parent )
        : q( parent ) {
    }

    Plasma::LineEdit *nameEdit;
    QList<Socket*> sockets;
    QMap<QString, OutputElement*> outputElements;
    QGraphicsLinearLayout* mainLay;
    bool failedToLoad;


private:
    BlackboxLoaderElement* q;
};


Konduit::Elements::BlackboxLoaderElement::BlackboxLoaderElement( QObject* parent, const QVariantList& args )
    : SwitchElement( parent, args ),
      d( new Private( this ))
{
    d->mainLay = new QGraphicsLinearLayout( Qt::Vertical, this);

    //load blackbox from the storage model
    QString name = pluginName();

    if ( !name.isEmpty() ) {
        Konduit::StorageModel *ksm = globalStorageModel();

        //add buttons
        qreal width = 70;//store plugin width, in case of buttons

        if ( Konduit::Element* e = ksm->loadKonduitBlackBox( name ) ) {
            foreach( Konduit::Element* elem, e->allConnectedElements() ) {

                if(elem->pluginName() == "konduit_input") {
                    InputElement *inpE = (InputElement*) elem;
                    //FIXME: crashes on more inputs with identical names
                    kDebug() << "adding input " << inpE->getName();
                    addPlug(inpE->getName());

                    //add intermediary socket and use that to interface with the external plug
                    Socket* s = createSocket( this, inpE->getName() );
                    d->sockets.append( s );
                    inpE->plug()->plugIn( s );
                }

                else if (elem->pluginName() == "konduit_output") {
                    OutputElement *outE = (OutputElement*) elem;
                    kDebug() << "adding output " << outE->getName();
                    addSocket(outE->getName());
                    d->outputElements.insert(outE->getName(), outE);//register name/outputelement pair
                }

                else if (elem->pluginName() == "konduit_datadump") {
                    kDebug() << "adding button for datadump";
                    //DataDumpElement* dde = (DataDumpElement*) elem;
                    Plasma::PushButton* startButton = new Plasma::PushButton( this );
                    startButton->setText( "Data Dump: Start" );
                    d->mainLay->addItem( startButton );
                    connect( startButton, SIGNAL( clicked() ), elem, SLOT( start() ) );
                    width = startButton->nativeWidget()->sizeHint().width() + 70;
                }

                else if (elem->pluginName() == "konduit_datatable") {
                    kDebug() << "adding button for datatable";
                    Plasma::PushButton* startButtonT = new Plasma::PushButton( this );
                    startButtonT->setText( "Data Table: Start" );
                    d->mainLay->addItem( startButtonT );
                    connect( startButtonT, SIGNAL( clicked() ), elem, SLOT( start() ) );
                    width = startButtonT->nativeWidget()->sizeHint().width() + 70;
                }

                else if (elem->pluginName() == "konduit_sendmail") {
                    kDebug() << "adding button for sendmail";
                    Plasma::PushButton* sendButton = new Plasma::PushButton( this );
                    sendButton->setText( "Send Emails" );
                    d->mainLay->addItem( sendButton );
                    connect( sendButton, SIGNAL( clicked() ), elem, SLOT( slotSendMails() ) );
                    width = sendButton->nativeWidget()->sizeHint().width() + 70;
                }
                
                 else if (elem->pluginName() == "konduit_fileopen") {
                    kDebug() << "adding button for fileopen";
                    Plasma::PushButton* openButton = new Plasma::PushButton( this );
                    openButton->setText( "File Open: Open" );
                    d->mainLay->addItem( openButton );
                    connect( openButton, SIGNAL( clicked() ), elem, SLOT( openFiles() ) );
                    width = openButton->nativeWidget()->sizeHint().width() + 70;
                }
                else if (elem->pluginName() == "konduit_datainserter") {
                    kDebug() << "adding button for datainserter";
                    Plasma::PushButton* openButton = new Plasma::PushButton( this );
                    openButton->setText( "Insert Data" );
                    openButton->setToolTip( "Inserts Data into the Nepomuk Repository" );
                    d->mainLay->addItem( openButton );
                    connect( openButton, SIGNAL( clicked() ), elem, SLOT( start() ) );
                    width = openButton->nativeWidget()->sizeHint().width() + 70;
                }
                
                else { //add button for blackbox  ( recursively )
                    QString constraint = QString("[X-KDE-PluginInfo-Name] == '%1'").arg(elem->pluginName());
                    KService::List services = KServiceTypeTrader::self()->query("Plasma/Applet", constraint);
                    if(!services.isEmpty()) {
                        KService::Ptr service = services.first();
                        if (service->library() == "konduit_blackboxloader") {
                            ////kDebug() << "looking for buttons in a blackbox";
                            BlackboxLoaderElement *elemBB = dynamic_cast<BlackboxLoaderElement*>( elem );
                            //Konduit::Elements::BlackboxLoader::Private *eD = elem->d;
                            QGraphicsLinearLayout *eLayout = elemBB->d->mainLay;
                            for(int i = 0; i<eLayout->count(); i++) {
                                //QGraphicsLayoutItem li = eLayout->itemAt(i);
                                Plasma::PushButton *btn = dynamic_cast<Plasma::PushButton*>( eLayout->itemAt(i) );
                                if (btn) {
                                    kDebug() << "adding button for blackbox";
                                    Plasma::PushButton* unkwnButton = new Plasma::PushButton( this );
                                    unkwnButton->setText( btn->text() );
                                    d->mainLay->addItem( unkwnButton );
                                    connect( unkwnButton, SIGNAL( clicked() ), btn->nativeWidget(), SLOT( click() ) );
                                    width = unkwnButton->nativeWidget()->sizeHint().width() + 70;
                                }
                            }
                        }
                    }
                }
            }
            d->failedToLoad = false;
        }

        else {
            //kDebug() << i18n( "Error loading konduit black box '%1'.").arg(name) ;
            d->failedToLoad = true;
            
        }

        setToolTip( pluginName() + " black box" );
        //setPreferredSize( QSizeF( width, ( countPlugs() + countSockets() ) * 45  + 25) );
        
        int c = countPlugs() + countSockets();
        setMinimumSize( QSizeF( width-25, c * 35 ) );
        resize( width, c == 0 ? 60 : c * 35  + 25 );
    }
}


Konduit::Elements::BlackboxLoaderElement::~BlackboxLoaderElement()
{
    delete d;
}


Soprano::StatementIterator Konduit::Elements::BlackboxLoaderElement::data( const Socket* s ) const
{
    if( d->sockets.contains( const_cast<Socket*>(s) ) ) { //if it is a hidden socket
        kDebug() << "retrieving data from socket " << s->name() << "through plug " << plug(s->name())->name();
        return plug(s->name())->data(); //retrieve data from corresponding plug
    }
    return d->outputElements[s->name()]->data( 0 );
}


QSizeF Konduit::Elements::BlackboxLoaderElement::contentsSizeHint( Qt::SizeHint, const QSizeF& ) const
{
    // FIXME: this is just a size that is based on the socket and plug sizes
    // in an ideal world they would not be fixed
    return size();//QSizeF( 30, ( countPlugs() + countSockets() ) * 30 );
}


void Konduit::Elements::BlackboxLoaderElement::paintInterface( QPainter* painter,
const QStyleOptionGraphicsItem* option,
const QRect& contentsRect )
{
  Q_UNUSED( option )
  Q_UNUSED( contentsRect )
  qreal left, top, right, bottom;
  getContentsMargins( &top, &left, &right, &bottom );
  
  painter->fillRect( QRectF( left, 
                             top, 
                             //contentsSizeHint(Qt::PreferredSize, QSizeF()).width(),
                             size().width() - left - right,
                             //contentsSizeHint(Qt::PreferredSize, QSizeF()).height()),
                             size().height() - top - bottom ),
                     QBrush(Qt::black));
}


bool Konduit::Elements::BlackboxLoaderElement::failedToLoad()
{
    return d->failedToLoad;
}


//#include "blackboxloaderelement.moc"
