/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "configurationdialog.h"
#include "configurationoption.h"
#include "configurationsetting.h"
#include "element.h"

#include <KEditListBox>
#include <KUrlRequester>
#include <KDebug>

#include <QtGui/QLabel>
#include <QtGui/QHBoxLayout>
#include <QtGui/QVBoxLayout>
#include <QtGui/QListWidget>
#include <QtGui/QLineEdit>


namespace {
    // FIXME: this is duplicated in Element
    QStringList valuesToStringList( const QVariantList& values ) {
        QStringList sl;
        for ( int i = 0; i < values.count(); ++i ) {
            const QVariant& v = values[i];
            if ( v.type() == QVariant::Url )
                sl << v.toUrl().toString();
            else
                sl << v.toString();
        }
        return sl;
    }

    QVariant variantFromString( const QString& s, QVariant::Type type ) {
        if ( type == QVariant::Url ) {
            return( QUrl( KUrl( s ) ) ); // KUrl makes sure we default to the file: scheme
        }
        else if ( type == QVariant::Int ) {
            return( s.toInt() );
        }
        else {
            return( s );
        }
    }
}

class Konduit::ConfigurationDialog::Private
{
public:
    Private( ConfigurationDialog* dlg )
        : q( dlg ) {
    }

    Element* element;

    QHash<QString, QWidget*> configWidgets;

    QLabel* createLabel( const ConfigurationOption& option );
    QWidget* createWidget( const ConfigurationOption& option, bool* big );
    QVariantList getValues( const ConfigurationOption& option );

private:
    ConfigurationDialog* q;
};


QLabel* Konduit::ConfigurationDialog::Private::createLabel( const ConfigurationOption& option )
{
    QLabel* label = new QLabel( option.label() + ':', q->mainWidget() );
    return label;
}


QWidget* Konduit::ConfigurationDialog::Private::createWidget( const ConfigurationOption& option, bool* big )
{
    // These are the widgets:
    // QLineEdit:     non-list string
    // KEditListBox:  any non-forced list
    //     custom editors:
    //     QSpinBox   integer
    //     KURLRequester : url
    // QListWidget:   any forced list
    // QSpinBox:      non-list integer

    *big = false;

    QList<QVariant> values = q->element()->optionSetting( option.name() ).values();

    // list selection
    // --------------
    if ( option.forceSuggestedValue() ) {
        QListWidget* listWidget = new QListWidget( q->mainWidget() );
        listWidget->setSelectionMode( option.isListOption() ? QAbstractItemView::MultiSelection : QAbstractItemView::SingleSelection );
        foreach( QVariant v, option.suggestedValues() ) {
            // QVariant::toString should work with any type
            listWidget->addItem( v.toString() );
        }


        // preselect the values
        QList<QListWidgetItem*> itemsToSelect;
        foreach( QVariant v, values ) {
            itemsToSelect += listWidget->findItems( v.toString(), Qt::MatchExactly );
        }

        if ( itemsToSelect.isEmpty() && option.defaultValue().isValid() ) {
            itemsToSelect = listWidget->findItems( option.defaultValue().toString(), Qt::MatchExactly );
        }

        foreach( QListWidgetItem* item, itemsToSelect ) {
            item->setSelected( true );
        }

        *big = true;
        return listWidget;
    }


    // line edit
    // --------------
    if ( option.type() == QVariant::String &&
         !option.isListOption() ) {
        QLineEdit* edit = new QLineEdit( q->mainWidget() );
        edit->setText( values.isEmpty() ? option.defaultValue().toString() : values.first().toString() );
        return edit;
    }
    else if ( option.type() == QVariant::Url &&
              !option.isListOption() ) {
        KUrlRequester* urlR = new KUrlRequester( q->mainWidget() );
        urlR->setUrl( values.isEmpty() ? option.defaultValue().toUrl() : values.first().toUrl() );
        return urlR;
    }


    // edit list box
    // ---------------
    if ( option.isListOption() ) {
        KEditListBox* box = 0;
        if ( option.type() == QVariant::Url ) {
            KUrlRequester* urlReq = new KUrlRequester();
            KEditListBox::CustomEditor customEditor( urlReq, urlReq->lineEdit() );
            box = new KEditListBox( option.label(), customEditor, q->mainWidget() );
        }
        else {
            box = new KEditListBox( option.label(), q->mainWidget() );
        }

        QVariant defaultV = option.defaultValue();
        if ( !values.isEmpty() ) {
            box->setItems( valuesToStringList( values ) );
        }
        else if ( defaultV.isValid() ) {
            box->setItems( valuesToStringList( QVariantList() << defaultV ) );
        }
        *big = true;
        return box;
    }

    Q_ASSERT( 0 );
    return 0;
}


QVariantList Konduit::ConfigurationDialog::Private::getValues( const ConfigurationOption& option )
{
    QWidget* configWidget = configWidgets[option.name()];

    // list selection
    // --------------
    if ( option.forceSuggestedValue() ) {
        QListWidget* listWidget = qobject_cast<QListWidget*>( configWidget );
        QList<QListWidgetItem*> items = listWidget->selectedItems();
        QVariantList values;
        foreach( QListWidgetItem* item, items ) {
            values << variantFromString( item->text(), option.type() );
        }
        return values;
    }

    // line edit
    // --------------
    if ( option.type() == QVariant::String &&
         !option.isListOption() ) {
        QLineEdit* edit = qobject_cast<QLineEdit*>( configWidget );
        return QVariantList() << QVariant( edit->text() );
    }
    else if ( option.type() == QVariant::Url &&
              !option.isListOption() ) {
        KUrlRequester* urlR = qobject_cast<KUrlRequester*>( configWidget );
        return QVariantList() << QVariant( urlR->url() );
    }


    // edit list box
    // ---------------
    if ( option.isListOption() ) {
        KEditListBox* box = qobject_cast<KEditListBox*>( configWidget );

        QVariantList vars;
        foreach( QString s, box->items() ) {
            vars << variantFromString( s, option.type() );
        }
        kDebug() << vars;
        return vars;
    }

    Q_ASSERT( 0 );
    return QVariantList();
}



Konduit::ConfigurationDialog::ConfigurationDialog( Element* element )
    : KDialog( 0 ),
      d( new Private( this ) )
{
    kDebug();
    d->element = element;

    setButtons( Ok|Cancel );
    setCaption( QString( "%1 Configuration" ).arg( element->name() ) );

    connect( this, SIGNAL( okClicked() ),
             this, SLOT( slotSave() ) );

    createGui();
}


Konduit::ConfigurationDialog::~ConfigurationDialog()
{
    delete d;
}


Konduit::Element* Konduit::ConfigurationDialog::element() const
{
    return d->element;
}


void Konduit::ConfigurationDialog::createGui()
{
    QGridLayout* mainLayout = new QGridLayout( mainWidget() );
    mainLayout->setMargin( 0 );

    QList<ConfigurationOption> options = element()->configurationOptions();
    int row = 0;
    Q_FOREACH( ConfigurationOption option, options ) {
        QLabel* label = d->createLabel( option );
        bool big = false;
        QWidget* editWidget = d->createWidget( option, &big );
        if ( big )
            label->setAlignment( Qt::AlignTop|Qt::AlignLeft );

        label->setBuddy( editWidget );

        mainLayout->addWidget( label, row, 0, 1, 1 );
        mainLayout->addWidget( editWidget, row, 1, 1, 1 );

        d->configWidgets.insert( option.name(), editWidget );

        ++row;
    }
}


void Konduit::ConfigurationDialog::slotSave()
{
    kDebug();

    QList<ConfigurationOption> options = element()->configurationOptions();
    Q_FOREACH( ConfigurationOption option, options ) {
        ConfigurationSetting setting;
        if ( option.isListOption() ) {
            setting.setValues( d->getValues( option ) );
        }
        else {
            setting.setValue( d->getValues( option ).first() );
        }
        element()->setOption( option.name(), setting );
    }
}

#include "configurationdialog.moc"
