/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "configurationoption.h"

#include <QtCore/QSharedData>


class Konduit::ConfigurationOption::Private : public QSharedData
{
public:
    Private( const QString& name_ = QString(),
             QVariant::Type type_ = QVariant::Invalid,
             const QVariant& defaultValue_ = QVariant() )
        : name( name_ ),
          type( type_ ),
          isListOption( false ),
          forceSuggestedValue( false ),
          defaultValue( defaultValue_ ) {
    }

    QString name;
    QVariant::Type type;
    bool isListOption;
    QList<QVariant> suggestedValues;
    bool forceSuggestedValue;
    QVariant defaultValue;
    QString toolTip;
    QString whatsThis;
    QString label;
};

Konduit::ConfigurationOption::ConfigurationOption()
    : d( new Private() )
{
}


Konduit::ConfigurationOption::ConfigurationOption( const QString& name, QVariant::Type type, const QVariant& defaultValue )
    : d( new Private() )
{
    setName( name );
    setType( type );
    setDefaultValue( defaultValue );
}


Konduit::ConfigurationOption::ConfigurationOption( const ConfigurationOption& other )
{
    d = other.d;
}


Konduit::ConfigurationOption::~ConfigurationOption()
{
}


void Konduit::ConfigurationOption::setLabel( const QString& label )
{
    d->label = label;
}


QString Konduit::ConfigurationOption::label() const
{
    return ( d->label.isEmpty() ? name() : d->label );
}


Konduit::ConfigurationOption& Konduit::ConfigurationOption::operator=( const ConfigurationOption& other )
{
    d = other.d;
    return *this;
}


QString Konduit::ConfigurationOption::name() const
{
    return d->name;
}


QVariant::Type Konduit::ConfigurationOption::type() const
{
    return d->type;
}


bool Konduit::ConfigurationOption::isListOption() const
{
    return d->isListOption;
}


QList<QVariant> Konduit::ConfigurationOption::suggestedValues() const
{
    return d->suggestedValues;
}


bool Konduit::ConfigurationOption::forceSuggestedValue() const
{
    return d->forceSuggestedValue;
}


QVariant Konduit::ConfigurationOption::defaultValue() const
{
    return d->defaultValue;
}


QString Konduit::ConfigurationOption::toolTip() const
{
    return d->toolTip;
}


QString Konduit::ConfigurationOption::whatsThis() const
{
    return d->whatsThis;
}


void Konduit::ConfigurationOption::setName( const QString& name )
{
    d->name = name;
}


void Konduit::ConfigurationOption::setType( QVariant::Type type )
{
    // FIXME: support more types
    Q_ASSERT( type != QVariant::Invalid );
    Q_ASSERT( type == QVariant::String ||
              type == QVariant::Url ||
              type == QVariant::Int );

    d->type = type;
}


void Konduit::ConfigurationOption::setIsListOption( bool b )
{
    d->isListOption = b;
}


void Konduit::ConfigurationOption::setSuggestedValues( const QList<QVariant>& list )
{
    Q_ASSERT( list.isEmpty() || list.first().type() == d->type );
    d->suggestedValues = list;
}


void Konduit::ConfigurationOption::addSuggestedValue( const QVariant& value )
{
    Q_ASSERT( value.type() == d->type );
    d->suggestedValues.append( value );
}


void Konduit::ConfigurationOption::setForceSuggestedValue( bool b )
{
    d->forceSuggestedValue = b;
}


void Konduit::ConfigurationOption::setDefaultValue( const QVariant& value )
{
    Q_ASSERT( !value.isValid() || value.type() == d->type );
    d->defaultValue = value;
}


void Konduit::ConfigurationOption::setToolTip( const QString& text )
{
    d->toolTip = text;
}


void Konduit::ConfigurationOption::setWhatsThis( const QString& text )
{
    d->whatsThis = text;
}
