/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "dataduplexer.h"
#include "socket.h"
#include "plug.h"

#include <Soprano/Statement>
#include <Soprano/StatementIterator>
#include <Soprano/IteratorBackend>

#include <QtCore/QQueue>
#include <QtCore/QDebug>
#include <QtGui/QPen>
#include <QtGui/QPainter>


namespace {
    class DuplexerStatementIteratorBackend;

    class StatementDuplexer : public QSharedData
    {
    public:
        StatementDuplexer( Soprano::StatementIterator it )
            : m_it( it ),
              m_fastBackend( 0 ),
              m_slowBackend( 0 ),
              m_closedBackend( 0 ) {
            qDebug() << "StatementDuplexer::StatementDuplexer()";
        }
        ~StatementDuplexer() {
            qDebug() << "StatementDuplexer::~StatementDuplexer()";
        }

        bool next( const DuplexerStatementIteratorBackend* db );
        Soprano::Statement current( const DuplexerStatementIteratorBackend* db ) const;
        void close( const DuplexerStatementIteratorBackend* db );

    private:
        QQueue<Soprano::Statement> m_statementQueue;
        Soprano::StatementIterator m_it;

        const DuplexerStatementIteratorBackend* m_fastBackend;
        const DuplexerStatementIteratorBackend* m_slowBackend;

        const DuplexerStatementIteratorBackend* m_closedBackend;
    };


    class DuplexerStatementIteratorBackend : public Soprano::IteratorBackend<Soprano::Statement>
    {
    public:
        DuplexerStatementIteratorBackend( StatementDuplexer* duplexer )
            : m_duplexer( duplexer ) {
            qDebug() << "DuplexerStatementIteratorBackend::DuplexerStatementIteratorBackend()";
        }
        ~DuplexerStatementIteratorBackend() {
            qDebug() << "DuplexerStatementIteratorBackend::~DuplexerStatementIteratorBackend()";
        }

        bool next();
        Soprano::Statement current() const;
        void close();

    private:
        // FIXME: sadly the ref counting does not work properly...
//        QSharedDataPointer<StatementDuplexer> m_duplexer;
        StatementDuplexer* m_duplexer;
    };


    bool StatementDuplexer::next( const DuplexerStatementIteratorBackend* db )
    {
        qDebug() << "StatementDuplexer::next" << db << m_fastBackend << m_slowBackend;
        // fast duplexer -> queue the current statement
        if ( !m_fastBackend || db == m_fastBackend ) {
            qDebug() << "StatementDuplexer::next: fastDuplexer" << db;
            m_fastBackend = db;
            if ( m_it.next() ) {
                m_statementQueue.enqueue( *m_it );
                return true;
            }
            else {
                return false;
            }
        }

        // no slow duplexer yet -> first call as slow, use queued statement
        else if ( !m_slowBackend ) {
            qDebug() << "StatementDuplexer::next: !slowDuplexer" << db;
            m_slowBackend = db;
            return !m_statementQueue.isEmpty();
        }

        // subsequent slow call -> advance in queue
        else {
            qDebug() << "StatementDuplexer::next: slowDuplexer" << db;
            // move to the next in the queue
            m_statementQueue.dequeue();

            // the slow one might catch up with the fast one
            if ( m_statementQueue.isEmpty() ) {
                m_slowBackend = 0;
                m_fastBackend = 0;
                return next( db );
            }
            else {
                return true;
            }
        }
    }


    Soprano::Statement StatementDuplexer::current( const DuplexerStatementIteratorBackend* db ) const
    {
        qDebug() << "StatementDuplexer::current" << db;
        if ( db == m_fastBackend ) {
            return *m_it;
        }
        else if ( !m_statementQueue.isEmpty() ) {
            return m_statementQueue.head();
        }
        else {
            return Soprano::Statement();
        }
    }


    void StatementDuplexer::close( const DuplexerStatementIteratorBackend* db )
    {
        if ( m_closedBackend && m_closedBackend != db ) {
            m_it.close();
        }
        else {
            m_closedBackend = db;
        }
    }


    bool DuplexerStatementIteratorBackend::next()
    {
        return m_duplexer->next( this );
    }


    Soprano::Statement DuplexerStatementIteratorBackend::current() const
    {
        return m_duplexer->current( this );
    }


    void DuplexerStatementIteratorBackend::close()
    {
        m_duplexer->close( this );
    }
}


class Konduit::Elements::DataDuplexer::Private
{
public:
    Private()
        : numUsed( 0 ),
          duplexer( 0 ) {
    }

    int numUsed;
    // FIXME: sadly the ref counting does not work properly...
//    QSharedDataPointer<StatementDuplexer> duplexer;
    StatementDuplexer* duplexer;
};


Konduit::Elements::DataDuplexer::DataDuplexer( QObject* parent, const QVariantList& args )
    : SwitchElement( parent, args ),
      d( new Private() )
{
    addPlug( "input" );
    addSocket( "data1" );
    addSocket( "data2" );
    setToolTip("Data Duplexer");
    resize( QSizeF(75, 120) );
    setMinimumSize( QSizeF(55, 100) );
}


Konduit::Elements::DataDuplexer::~DataDuplexer()
{
    delete d;
}


Konduit::Socket* Konduit::Elements::DataDuplexer::firstSocket() const
{
    return socket( "data1" );
}


Konduit::Socket* Konduit::Elements::DataDuplexer::secondSocket() const
{
    return socket( "data2" );
}


Soprano::StatementIterator Konduit::Elements::DataDuplexer::data( const Socket* s ) const
{
    Q_UNUSED( s )

    if ( !plug( "input" )->socket() ) {
        qDebug() << "DataDuplexer::data: not connected to any input";
        return Soprano::StatementIterator();
    }

    if ( d->numUsed == 2 || d->numUsed == 0 ) {
        qDebug() << "DataDuplexer::data: need new StatementDuplexer";
        d->numUsed = 0;
        d->duplexer = new StatementDuplexer( plug( "input" )->socket()->data() );
    }

    ++d->numUsed;
    return new DuplexerStatementIteratorBackend( d->duplexer );
}


QSizeF Konduit::Elements::DataDuplexer::contentsSizeHint( Qt::SizeHint which, const QSizeF& constraint ) const
{
    Q_UNUSED( which );
    Q_UNUSED( constraint );
    // FIXME: this is just a size that is based on the socket and plug sizes
    // in an ideal world they would not be fixed
    return QSizeF( 40, 90 );
}


void Konduit::Elements::DataDuplexer::paintInterface( QPainter* painter,
                                                      const QStyleOptionGraphicsItem* option,
                                                      const QRect& contentsRect )
{
    Q_UNUSED( option );
    painter->save();
    painter->translate( contentsRect.left(), contentsRect.top() );

    // we draw some small lines to indicate the duplexing
    painter->setPen( QPen( Qt::white, 2 ) );
    painter->drawLine( 0, 10, 20, 10 );
    painter->drawLine( 0, 40, 20, 40 );
    painter->drawLine( 0, 70, 20, 70 );
    painter->drawLine( 20, 10, 20, 70 );

    painter->drawLine( 30, 36, 30, 50 );

    QPainterPath path( QPointF( 26, 36 ) );
    path.lineTo( 30, 30 );
    path.lineTo( 34, 36 );
    path.lineTo( 26, 36 );
    painter->fillPath( path, Qt::white );

    painter->restore();
}
