/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "datamerger.h"
#include "socket.h"
#include "plug.h"

#include <Soprano/Statement>
#include <Soprano/StatementIterator>
#include <Soprano/IteratorBackend>

#include <QtCore/QQueue>
#include <QtCore/QDebug>
#include <QtGui/QPen>
#include <QtGui/QPainter>


namespace {
    class MergerStatementIteratorBackend : public Soprano::IteratorBackend<Soprano::Statement>
    {
    public:
        MergerStatementIteratorBackend( const Soprano::StatementIterator& it1, const Soprano::StatementIterator& it2 )
            : Soprano::IteratorBackend<Soprano::Statement>(),
              m_it1( it1 ),
              m_it2( it2 ),
              m_first( false ) {
        }

        bool next() {
            if ( m_it1.next() ) {
                m_first = true;
                return true;
            }
            else {
                m_first = false;
                return m_it2.next();
            }
        }

        Soprano::Statement current() const {
            if ( m_first ) {
                return m_it1.current();
            }
            else {
                return m_it2.current();
            }
        }

        void close() {
            m_it1.close();
            m_it2.close();
        }

    private:
        Soprano::StatementIterator m_it1;
        Soprano::StatementIterator m_it2;
        bool m_first;
    };
}


class Konduit::Elements::DataMerger::Private
{
public:
};


Konduit::Elements::DataMerger::DataMerger( QObject* parent, const QVariantList& args )
    : SwitchElement( parent, args ),
      d( 0 )
{
    addPlug( "input1" );
    addPlug( "input2" );
    addSocket( "data" );
    setToolTip("Data Merger");
    resize( QSizeF(75, 120) );
    setMinimumSize( QSizeF(55, 100) );
}


Konduit::Elements::DataMerger::~DataMerger()
{
    delete d;
}


Soprano::StatementIterator Konduit::Elements::DataMerger::data( const Socket* s ) const
{
    Q_UNUSED( s )

    if ( !plug( "input1" )->socket() || !plug( "input2" )->socket() ) {
        qDebug() << "DataMerger::data: not connected to any input";
        return Soprano::StatementIterator();
    }

    return new MergerStatementIteratorBackend( plug( "input1" )->socket()->data(),
                                               plug( "input2" )->socket()->data() );
}


QSizeF Konduit::Elements::DataMerger::contentsSizeHint( Qt::SizeHint which, const QSizeF& constraint ) const
{
    //kDebug() << which << constraint;
    Q_UNUSED( which );
    Q_UNUSED( constraint );

    // FIXME: this is just a size that is based on the socket and plug sizes
    // in an ideal worls they would not be fixed
    return QSizeF( 40, 90 );
}


void Konduit::Elements::DataMerger::paintInterface( QPainter* painter,
                                                    const QStyleOptionGraphicsItem* option,
                                                    const QRect& contentsRect )
{
    Q_UNUSED( option );
    painter->save();
    painter->translate( contentsRect.left(), contentsRect.top() );

    // we draw some small lines to indicate the duplexing
    painter->setPen( QPen( Qt::white, 2 ) );
    painter->drawLine( 0, 10, 20, 10 );
    painter->drawLine( 0, 40, 20, 40 );
    painter->drawLine( 0, 70, 20, 70 );
    painter->drawLine( 20, 10, 20, 70 );

    painter->drawLine( 30, 36, 30, 50 );

    QPainterPath path( QPointF( 26, 50 ) );
    path.lineTo( 30, 56 );
    path.lineTo( 34, 50 );
    path.lineTo( 26, 50 );
    painter->fillPath( path, Qt::white );

    painter->restore();
}
