/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "element.h"
#include "sink.h"
#include "source.h"
#include "plug.h"
#include "socket.h"
#include "manager.h"
#include "konduitstoragemodel.h"
#include "configurationdialog.h"
#include "pluginicon.h"
#include "resizeicon.h"
#include "inputelement.h"
#include "outputelement.h"


#include <QtCore/QHash>
#include <QtCore/QPair>
#include <QtCore/QTimer>
#include <QtGui/QGraphicsScene>
#include <QtGui/QPainter>
#include <QtGui/QDialog>
#include <QtGui/QGraphicsSceneContextMenuEvent>
#include <QGraphicsLinearLayout>
#include <QInputDialog>
#include <QIcon>
#include <QLabel>
#include <QSizePolicy>

#include <KIcon>
#include <KMenu>
#include <kfiledialog.h>
#include <kio/netaccess.h>
#include <KLocale>
#include <KAction>
#include <KInputDialog>
#include <KMessageBox>
#include <KStandardAction>
#include <KServiceTypeTrader>

#include <plasma/widgets/lineedit.h>
#include <plasma/widgets/label.h>
#include <plasma/widgets/pushbutton.h>

#include <Soprano/Client/DBusClient>
#include <Soprano/Client/DBusModel>
#include <Soprano/Util/DummyModel>
#include <Soprano/StatementIterator>

K_GLOBAL_STATIC(Konduit::StorageModel, globalKsm);

namespace {
    // FIXME: this is duplicated in ConfigurationDialog
    QStringList valuesToStringList( const QVariantList& values ) {
        QStringList sl;
        for ( int i = 0; i < values.count(); ++i ) {
            const QVariant& v = values[i];
            if ( v.type() == QVariant::Url )
                sl << v.toUrl().toString();
            else
                sl << v.toString();
        }
        return sl;
    }
}


class Konduit::Element::Private
{
public:
    Private()
        : saveAction( 0 ), saveBlackBoxAction( 0 ), openBlackBoxAction( 0 ) {
        groupId = Manager::instance()->generateGroupId();
        isDragging = false;
    }

    QHash<QString, QPair<ConfigurationOption, ConfigurationSetting> > settings;
    QHash<QString, QGraphicsProxyWidget*> optionWidgets;

    int groupId;

    QAction* saveAction;
    QAction* saveBlackBoxAction;
    QAction* openBlackBoxAction;
    QString crtGroupName; //to pass the value of saved blackboxes between functions
    QString crtGroupComment; //to pass the comment of saved blackboxes between functions
    QTimer* timer; //for periodical checking if saved blackbox was found Plasma
    //Plasma::Label* label; // for putting a label on each element
    PluginIcon *pluginIcon;
    ResizeIcon *resizeIcon;

    QPointF startDragPos;
    bool isDragging;
    QSizeF originalSize;

    bool isStandardGui;
};


Konduit::Element::Element( QGraphicsItem* parent, const QString& serviceId, uint appletId )
    : Plasma::Applet( parent, serviceId, appletId ),
      d( new Private() )
{
    initialize();
}


Konduit::Element::Element( QObject* parent, const QVariantList& args )
    : Plasma::Applet( parent, args ),
      d( new Private() )
{
    initialize();
}

void Konduit::Element::initialize()
{
    setBackgroundHints( DefaultBackground );
    setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
    setFlag( QGraphicsItem::ItemIsMovable, true );

    // a little hack to make sure the margins are valid when calling updatePositions
    QTimer::singleShot( 0, this, SLOT( updatePositions() ) );

    d->pluginIcon = new PluginIcon( this );
    d->resizeIcon = new ResizeIcon( this );
    d->isStandardGui = false;
}


Konduit::Element::~Element()
{
    delete d;
}


int Konduit::Element::groupId() const
{
    return d->groupId;
}


bool Konduit::Element::isSink() const
{
    return dynamic_cast<const Sink*>( this );
}


bool Konduit::Element::isSource() const
{
    return dynamic_cast<const Source*>( this );
}


static Soprano::Model* s_model = 0;

void Konduit::Element::setTestModel( Soprano::Model* model )
{
    s_model = model;
}


Soprano::Model* Konduit::Element::nepomukMainModel() const
{
    // we use a dummy test model here
    if ( !s_model ) {
        static Soprano::Client::DBusClient client( "org.kde.NepomukServer" );
        s_model = client.createModel( "main" );
    }

    if ( !s_model ) {
        s_model = new Soprano::Util::DummyModel();
    }

    return s_model;
}


QList<Konduit::ConfigurationOption> Konduit::Element::configurationOptions() const
{
    // Should we cache this?
    QList<ConfigurationOption> options;
    for ( QHash<QString, QPair<ConfigurationOption, ConfigurationSetting> >::iterator it = d->settings.begin();
          it != d->settings.end(); ++it ) {
        options.append( it.value().first );
    }
    return options;
}


void Konduit::Element::setOption( const QString& name, const ConfigurationSetting& setting )
{
    if ( d->settings.contains( name ) ) {
        kDebug() << "(Konduit::Element) setting option" << name << "to" << setting.values();
        d->settings[name].second = setting;
        emit configurationValueChanged( name );
    }
}


// FIXME: we should handle the default value here. Maybe even remove this aweful ConfigurationSetting class
Konduit::ConfigurationSetting Konduit::Element::optionSetting( const QString& name ) const
{
    if ( d->settings.contains( name ) ) {
        return d->settings[name].second;
    }
    else {
        return ConfigurationSetting();
    }
}


void Konduit::Element::addConfigurationOption( const ConfigurationOption& option )
{
    Q_ASSERT( !option.name().isEmpty() );
    d->settings[option.name()] = qMakePair<ConfigurationOption, ConfigurationSetting>( option, ConfigurationSetting( option.defaultValue() ) );
}


void Konduit::Element::createStandardGui()
{
    setHasConfigurationInterface( true );
    connect( this, SIGNAL( configurationValueChanged( const QString& ) ),
             this, SLOT( updateStandardGui() ) );

    // FIXME: simpler GUI would be: display all settings with labels and create a default config dialog
    // FIXME: connect to the configurationValueChanged signal

    //kDebug() << "Creating standard GUI";
    d->isStandardGui = true;

    QGraphicsLinearLayout* mainLay = new QGraphicsLinearLayout( Qt::Vertical, this );
    //setLayout( mainLay ); // only used for memory management

    QList<ConfigurationOption> options = configurationOptions();
    Q_FOREACH( ConfigurationOption option, options ) {
        if ( option.type() == QVariant::String || option.type() == QVariant::Url ) {
            kDebug() << "Creating string config GUI";

            QGraphicsLinearLayout* line = new QGraphicsLinearLayout( mainLay );

            Plasma::Label* label = new Plasma::Label( this );
            label->nativeWidget()->setStyleSheet(" QLabel { background-color:#ffffff; color:#000000; text-align: left } " );
            label->setText( option.name() + ":" );

            Plasma::Label* edit = new Plasma::Label( this );
            edit->nativeWidget()->setStyleSheet(" QLabel { background-color:#ffffff; color:#222222; text-align: left } " );
            edit->setText( option.defaultValue().toString() );
            edit->nativeWidget()->setFrameStyle( QFrame::Box );

            line->addItem( label );
            line->addItem( edit );
            line->setStretchFactor( edit, 1 );

            mainLay->addItem( line );
            line->activate();
            mainLay->activate();

            d->optionWidgets.insert( option.name(), edit );
        }
        else {
            // FIXME: handle other types
            Q_ASSERT( 0 );
        }
    }
}


void Konduit::Element::updateStandardGui()
{
    if(!d->isStandardGui) { //elements having ConfigurationOptions but behaving in a non-standard way can't perform these operations
        return;
    }

    qreal left, top, right, bottom;
    getContentsMargins( &top, &left, &right, &bottom );

    QList<ConfigurationOption> options = configurationOptions();
    Q_FOREACH( ConfigurationOption option, options ) {
        QGraphicsProxyWidget* configWidget = d->optionWidgets[option.name()];
        if ( option.type() == QVariant::String || option.type() == QVariant::Url ) {
            Plasma::Label* label = qobject_cast<Plasma::Label*>( configWidget );

            label->nativeWidget()->setMaximumWidth( size().width() - label->pos().x() - right - left );//don't let the text resize the label
            label->nativeWidget()->setMaximumHeight( ( size().height() - label->pos().y() ) / options.count());//don't let the text resize the label

            label->setText( valuesToStringList( optionSetting( option.name() ).values() ).join( "\n" ) );
            label->nativeWidget()->setToolTip( label->text() );//for large texts
            label->update( label->boundingRect() );
        }
    }

    updateGeometry();
}


void Konduit::Element::showKonduitConfigurationInterface()
{
    // we are showing a default interface here
    ConfigurationDialog dlg( this );
    if (QDialog::Accepted == dlg.exec()) {
        updateStandardGui();
    }
}


QVariant Konduit::Element::itemChange( GraphicsItemChange change, const QVariant& value )
{
    if ( change == QGraphicsItem::ItemPositionHasChanged ) {
        updatePositions();
    }
    else if ( change == QGraphicsItem::ItemSceneChange ) {
        QGraphicsScene* newScene = value.value<QGraphicsScene*>();

        Elements::OutputElement* outputElement = dynamic_cast<Konduit::Elements::OutputElement*> (this);
        Elements::InputElement* inputElement = dynamic_cast<Konduit::Elements::InputElement*> (this);

        if ( isSink() && inputElement == 0 ) { //hide plug for InputElement
            QList<Plug*> plugs = dynamic_cast<Sink*>( this )->allPlugs();
            Q_FOREACH( Plug* plug, plugs ) {
                if ( scene() ) {
                    scene()->removeItem( plug );
                }
                if ( newScene ) {
                    newScene->addItem( plug );
                }
            }
        }
        if ( isSource() && outputElement == 0 ) {//hide socket for OutputElement
            QList<Socket*> sockets = dynamic_cast<Source*>( this )->allSockets();
            Q_FOREACH( Socket* socket, sockets ) {
                if ( scene() ) {
                    scene()->removeItem( socket );
                }
                if ( newScene ) {
                    newScene->addItem( socket );
                }
             }
        }

        //add pluginIcon in all cases
                if ( scene() ) {
                    scene()->removeItem( d->pluginIcon );
                }
                if ( newScene ) {
                    newScene->addItem( d->pluginIcon );
                }
        //add resizeIcon in all cases
                if ( scene() ) {
                    scene()->removeItem( d->resizeIcon );
                }
                if ( newScene ) {
                    newScene->addItem( d->resizeIcon );
                }

        updatePositions();
    }

    return Plasma::Applet::itemChange( change, value );
}


void Konduit::Element::updatePositions()
{
    QPointF pos = scenePos();
    int i = 0;

    qreal left, top, right, bottom;
    getContentsMargins( &top, &left, &right, &bottom );

    Elements::OutputElement* outputElement = dynamic_cast<Konduit::Elements::OutputElement*> (this);
    Elements::InputElement* inputElement = dynamic_cast<Konduit::Elements::InputElement*> (this);

    if ( isSource() && outputElement == 0 ) {//hide plug for InputElement
        QList<Socket*> sockets = dynamic_cast<Source*>( this )->allSockets();
        Q_FOREACH( Socket* socket, sockets ) {
            QRectF rect = socket->sceneBoundingRect();
            socket->setPos( QPointF( pos.x() - rect.size().width() + left, pos.y() + i*30 + top ) );
            ++i;
        }
    }

    if ( isSink() && inputElement == 0 ) {//hide socket for OutputElement
        QList<Plug*> plugs = dynamic_cast<Sink*>( this )->allPlugs();
        Q_FOREACH( Plug* plug, plugs ) {
            QRectF rect = plug->sceneBoundingRect();
            plug->setPos( QPointF( pos.x() - rect.size().width() + left, pos.y() + i*30 + top ) );
            ++i;
        }
    }

    QRectF rect = d->pluginIcon->sceneBoundingRect();
    d->pluginIcon->setPos( QPointF( pos.x() - rect.size().width() + left + 20 , pos.y() ) );

    rect = d->resizeIcon->sceneBoundingRect();
    d->resizeIcon->setPos( QPointF( pos.x() + size().width() - rect.size().width(),
                                    pos.y() + size().height() - rect.size().height() ) );

}


void Konduit::Element::paintInterface( QPainter* painter,
                                       const QStyleOptionGraphicsItem* option,
                                       const QRect& contentsRect )
{
    painter->fillRect( contentsRect, Qt::white );
    // FIXME: strangely the lower right point is not in the drawable rect anymore
//     QRect rect( contentsRect );
//     rect.setWidth( rect.width()-1 );
//     rect.setHeight( rect.height()-1 );
//     painter->drawRect( rect );
//     painter->drawText( rect, Qt::AlignHCenter, name() );
    Applet::paintInterface( painter, option, contentsRect );
}


void Konduit::Element::slotPlugStateChanged( Plug* oldPlug, Plug* newPlug )
{
    if ( newPlug ) {
        moveToGroup( dynamic_cast<const Element*>( newPlug->sink() )->groupId() );
    }
    else if ( oldPlug ) {
        moveToGroup( Manager::instance()->generateGroupId() );
    }

    emit plugStateChanged();
}


void Konduit::Element::slotPlugStateChanged( Socket* oldSocket, Socket* newSocket )
{
    Q_UNUSED( oldSocket );
    Q_UNUSED( newSocket );

    emit plugStateChanged();
}


bool Konduit::Element::isFullyPlugged() const
{
    return isFullyPlugged( 0 );
}


bool Konduit::Element::isFullyPlugged( const Element* other ) const
{
    if ( isSource() ) {
        QList<Socket*> sockets = dynamic_cast<const Source*>( this )->allSockets();
        Q_FOREACH( Socket* socket, sockets ) {
            if ( !socket->plug() ) {
                return false;
            }
            else if ( ( const Element* )socket->plug()->sink() != other ) {
                if ( !dynamic_cast<const Element*>( socket->plug()->sink() )->isFullyPlugged( this ) ) {
                    return false;
                }
            }
        }
    }

    if ( isSink() ) {
        QList<Plug*> plugs = dynamic_cast<const Sink*>( this )->allPlugs();
        Q_FOREACH( Plug* plug, plugs ) {
            if ( !plug->socket() ) {
                return false;
            }
            else if ( ( const Element* )plug->socket()->source() != other ) {
                if ( !dynamic_cast<const Element*>( plug->socket()->source() )->isFullyPlugged( this ) ) {
                    return false;
                }
            }
        }
    }
    return true;
}


QList<Konduit::Element*> Konduit::Element::connectedElements() const
{
    QList<Element*> elements;

    if ( isSource() ) {
        QList<Socket*> sockets = dynamic_cast<const Source*>( this )->allSockets();
        Q_FOREACH( Socket* socket, sockets ) {
            if ( socket->plug() && !elements.contains( dynamic_cast<Element*>( socket->plug()->sink() ) ) ) {
                elements.append( dynamic_cast<Element*>( socket->plug()->sink() ) );
            }
        }
    }

    if ( isSink() ) {
        QList<Plug*> plugs = dynamic_cast<const Sink*>( this )->allPlugs();
        Q_FOREACH( Plug* plug, plugs ) {
            if ( plug->socket()  && !elements.contains( dynamic_cast<Element*>( plug->socket()->source() ) ) ) {
                elements.append( dynamic_cast<Element*>( plug->socket()->source() ) );
            }
        }
    }

    return elements;
}


QList<Konduit::Element*> Konduit::Element::allConnectedElements() const
{
    QSet<Element*> elements = connectedElements().toSet();

    // the most trivial way of traversing a graph!
    int cnt = 0;
    do {
        cnt = elements.count();
        foreach( Element* e, elements ) {
            elements += e->connectedElements().toSet();
        }
    } while ( cnt < elements.count() );

    return elements.toList();
}


void Konduit::Element::moveToGroup( int groupId )
{
    d->groupId = groupId;

    Manager::instance()->updateGrouping( this );

    // propagate the change
    if ( isSource() ) {
        QList<Socket*> sockets = dynamic_cast<Source*>( this )->allSockets();
        Q_FOREACH( Socket* socket, sockets ) {
            if ( socket && socket->plug() ) {
                Element* element = dynamic_cast<Element*>( socket->plug()->sink() );
                if ( element && element->groupId() != groupId ) {
                    element->moveToGroup( groupId );
                }
            }
        }
    }
    if ( isSink() ) {
        QList<Plug*> plugs = dynamic_cast<Sink*>( this )->allPlugs();
        Q_FOREACH( Plug* plug, plugs ) {
            if ( plug && plug->socket() ) {
                Element* element = dynamic_cast<Element*>( plug->socket()->source() );
                if ( element && element->groupId() != groupId ) {
                    element->moveToGroup( groupId );
                }
            }
        }
    }
}


void Konduit::Element::contextMenuEvent( QGraphicsSceneContextMenuEvent* event )
{
    KMenu desktopMenu;

    if (hasConfigurationInterface()) {
        QAction* configureApplet = new QAction(i18n("%1 &Settings...", name()), &desktopMenu);
        configureApplet->setIcon(KIcon("configure"));
        connect(configureApplet, SIGNAL(triggered(bool)),
                this, SLOT(showKonduitConfigurationInterface()));
        desktopMenu.addAction(configureApplet);
    }

    QAction* closeApplet = new QAction(i18n("&Remove this %1", name()), &desktopMenu);
    closeApplet->setIcon(KIcon("edit-delete"));
    connect(closeApplet, SIGNAL(triggered(bool)),
            //this, SLOT(deleteLater()));
             this, SLOT(destroy()));//"it will be removed nicely and deleted"
    desktopMenu.addAction(closeApplet);

    QList<QAction*> actions = contextActions();
    if (!actions.isEmpty()) {
        desktopMenu.addSeparator();
        foreach(QAction* action, actions) {
            desktopMenu.addAction(action);
        }
    }

    event->accept();

    desktopMenu.exec(event->screenPos());
}


QSizeF Konduit::Element::sizeHint( Qt::SizeHint which, const QSizeF& constraint ) const
{
    // FIXME: just like the plug and socket positioning this is stupid
    int cnt = 0;
    if ( isSource() ) {
        cnt += dynamic_cast<const Source*>( this )->allSockets().count();
    }
    if ( isSink() ) {
        cnt += dynamic_cast<const Sink*>( this )->allPlugs().count();
    }

    QSizeF size = Applet::sizeHint( which, constraint );
    size.setHeight( qMax( size.height(), qreal( cnt * 30 ) ) );

    // handle contents size hint
    qreal left, top, right, bottom;
    getContentsMargins( &top, &left, &right, &bottom );
    QSizeF pcs = contentsSizeHint( which, constraint );
    pcs.setWidth( pcs.width() + left + right );
    pcs.setHeight( pcs.height() + top + bottom );
    size = size.expandedTo( pcs );

    //kDebug() << size << "with" << cnt;

    return size;
}


QSizeF Konduit::Element::contentsSizeHint( Qt::SizeHint, const QSizeF& ) const
{
    return QSizeF();
}


void Konduit::Element::slotSaveGroup()
{
    if ( 1 || isFullyPlugged() ) {
        // We need to save:
        // 1. the elements: mainly just the type
        // 2. the connections (<element uri> konduit:connected <element uri>)
        // 3. named graph represents one group
        //    with unique name
        while ( 1 ) {
            QString groupName = KInputDialog::getText( 0, i18n( "Please choose a name for the stored konduit." ) );
            if ( groupName.isEmpty() ) {
                break;
            }
            Konduit::StorageModel *ksm = globalStorageModel();
            QUrl konduitUri = ksm->findKonduit( groupName );
            if ( konduitUri.isEmpty() ||
                 KMessageBox::questionYesNo( 0, i18n( "Should the Konduit '%1' be overwritten.", groupName ) ) == KMessageBox::Yes ) {
                kDebug() << "Saved Konduit under URI" << ksm->saveKonduit( this, groupName );
                KMessageBox::information( 0, i18n( "Konduit saved as '%1'.",  groupName ) );
                break;
            }
        }

    }
    else {
        KMessageBox::sorry( 0, i18n( "Only fully connected konduits can be saved." ) );
    }
}

void Konduit::Element::slotSaveBlackBox()
{
    if ( 1 || isFullyPlugged() ) {
        // We need to save:
        // 1. the elements: mainly just the type
        // 2. the connections (<element uri> konduit:connected <element uri>)
        // 3. named graph represents one group
        //    with unique name
        while ( 1 ) {
            Konduit::StorageModel *ksm = globalStorageModel();

            if( !ksm->validateBlackBox( this ) ) { //if blackbox not valid
                KMessageBox::sorry( 0, i18n( "Invalid blackbox!\n Please check input and output names,\n and disconnected elements." ) );
                break;
            }

            QString groupName = KInputDialog::getText( 0, i18n( "Please choose a name for the stored blackbox:" ) );
            if ( groupName.isEmpty() ) {
                break;
            }
            QUrl konduitUri = ksm->findKonduitBlackBox( groupName );

            if ( konduitUri.isEmpty() ||
                 KMessageBox::questionYesNo( 0, i18n( "Should the blackbox '%1' be overwritten.", groupName ) ) == KMessageBox::Yes ) {

            QString comment = KInputDialog::getText( 0, i18n( "Please enter comments for the blackbox:" ) );
            if ( comment.isEmpty() ) {
                break;
            }

            //display categories
            QStringList categories = Konduit::Element::listAppletCategoriesShort();
            
            bool ok = false;
            QString category = QInputDialog::getItem(0, tr("Please choose (or enter) category:"),
                                          tr("Category:"), categories, 0, true, &ok, 0);
            if ( category.isEmpty() || ok == false) {
                break;
            }
            
            if( !category.contains( "Konduit Elements - " )) {
                category.prepend( "Konduit Elements - " );
            }

            QUrl s = ksm->saveKonduitBlackBox( this, groupName, comment, category );

                if(!s.isEmpty()) {
                    kDebug() << "Saved Konduit under URI" << s;
                    KMessageBox::information( 0, i18n( "Blackbox saved as '%1'.",  groupName ) );
                    d->crtGroupName = groupName;
                    d->crtGroupComment = comment;
                    emit( blackBoxSaved( groupName ) );
                    break;
                }

                else {
                    KMessageBox::sorry( 0, i18n( "Error saving blackbox" ) );
                    break;
                }

            }
        }

    }
    else {
        KMessageBox::sorry( 0, i18n( "Only fully connected konduits can be saved." ) );
    }
}

void Konduit::Element::slotOpenBlackBox( )
{
    emit openBlackBox( name() );
}

QList<QAction*> Konduit::Element::contextActions()
{

    QList<QAction*> actionList;

    if ( !d->saveAction ) {
        d->saveAction = KStandardAction::save( this, SLOT( slotSaveGroup() ), this );
    }
    actionList << d->saveAction;


    if ( !d->saveBlackBoxAction ) {
        d->saveBlackBoxAction = new QAction(QIcon(), tr("Save &black box..."), this);
        //d->saveBlackBoxAction->setShortcut(tr("Ctrl+B"));
        d->saveBlackBoxAction->setStatusTip(tr("Save black box"));
        connect(d->saveBlackBoxAction, SIGNAL(triggered()), this, SLOT(slotSaveBlackBox()));
    }
    actionList << d->saveBlackBoxAction;

    if ( !d->openBlackBoxAction) {
        d->openBlackBoxAction = KStandardAction::open( this, SLOT( slotOpenBlackBox() ), this );
        d->openBlackBoxAction->setText( tr("Open this black box") );
        d->openBlackBoxAction->setShortcut( 0 );
    }

    //FIXME: use type reflection here. Something like dynamic_cast<Konduit::Elements::BlackboxLoader*>(this)
    if( !pluginName().startsWith("konduit_" )) {//only add this if it is a black box
        actionList << d->openBlackBoxAction;
    }

    return actionList;//QList<QAction*>() << d->saveAction << d->saveBlackBoxAction << d->openBlackBoxAction;
}

void Konduit::Element::hoverMoveEvent ( QGraphicsSceneHoverEvent * event )
{
    QPoint ep = event->pos().toPoint();
    QPoint rp = d->resizeIcon->pos().toPoint();
    QPoint p = pos().toPoint();
    QSizeF rs = d->resizeIcon->sceneBoundingRect().size();

    //kDebug() << ep << " , " << rp << " , " << p;

    //show resizeIcon when hovered on top of it
    if( ep.x() > rp.x() - p.x() && ep.x() < rp.x() - p.x() + rs.width() &&
        ep.y() > rp.y() - p.y() && ep.y() < rp.y() - p.y() + rs.height() ) {
        d->resizeIcon->setVisible( true );
    }
    else {
        d->resizeIcon->setVisible( false );
    }
}


void Konduit::Element::hoverLeaveEvent ( QGraphicsSceneHoverEvent * event )
{
    Q_UNUSED( event );
    d->resizeIcon->setVisible( false );
}

void Konduit::Element::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    QPoint ep = event->pos().toPoint();
    QPoint rp = d->resizeIcon->pos().toPoint();
    QPoint p = pos().toPoint();
    QSizeF rs = d->resizeIcon->sceneBoundingRect().size();

    //start resizing if on top of the resizeIcon
    if( ep.x() > rp.x() - p.x() && ep.x() < rp.x() - p.x() + rs.width() &&
        ep.y() > rp.y() - p.y() && ep.y() < rp.y() - p.y() + rs.height() ) {
        d->isDragging = true;
        d->startDragPos = event->pos();
        d->originalSize = size();
    }
    else {
        QGraphicsWidget::mousePressEvent( event );
    }
}


void Konduit::Element::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
     d->startDragPos = QPointF(0, 0);
     d->isDragging = false;
     d->originalSize = size();
     QGraphicsWidget::mouseReleaseEvent( event );
}


void Konduit::Element::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    if(!d->isDragging) {
        QGraphicsWidget::mouseMoveEvent( event );
        return;
    }

    QPointF newPos = event->pos();

    QSizeF oldSize = d->originalSize;
    QSizeF newSize( oldSize.width() + newPos.x() - d->startDragPos.x(),
                    oldSize.height() + newPos.y() - d->startDragPos.y() );

    resize( newSize );
    updatePositions();
    updateStandardGui();
}

Konduit::StorageModel *Konduit::Element::globalStorageModel()
{
    return globalKsm;
}

QStringList Konduit::Element::listAppletCategories()
{
    QString constraint = QString("exist [X-KDE-PluginInfo-Category]");
    KService::List offers = KServiceTypeTrader::self()->query("Plasma/Applet", constraint);
    
    QStringList categories;
    foreach (const KService::Ptr &applet, offers) {
        QString appletCategory = applet->property("X-KDE-PluginInfo-Category").toString();
        if(appletCategory.contains("Konduit Elements - ") && !categories.contains(appletCategory)) {
            categories << appletCategory;
        }
    }
    
    return categories;
}

QStringList Konduit::Element::listAppletCategoriesShort()
{
    QStringList categories;
    foreach (QString s, listAppletCategories()) {
            s.replace("Konduit Elements - ", "");
            categories << s;
        }
    return categories;
}

#include "element.moc"
