/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_ELEMENT_H_
#define _KONDUIT_ELEMENT_H_

#include <Plasma/Applet>
#include <plasma/widgets/label.h>

#include "konduit_export.h"

#include "configurationoption.h"
#include "configurationsetting.h"

#include <QtCore/QList>

class QSizeF;

namespace Soprano {
    class Model;
}


namespace Konduit {

    class Plug;
    class Socket;
    class StorageModel;

    /**
     * The base class for all Konduit elements.
     */
    class KONDUIT_EXPORT Element : public Plasma::Applet
    {
        Q_OBJECT

    public:
        virtual ~Element();

        bool isSink() const;
        bool isSource() const;

        QList<ConfigurationOption> configurationOptions() const;
        void setOption( const QString& name, const ConfigurationSetting& setting );
        ConfigurationSetting optionSetting( const QString& name ) const;

        /**
         * Connected elements are in the same group. So each group represents one 
         * connected graph.
         */
        int groupId() const;

        /**
         * Recursively check if the Element is fully plugged, i.e. if all plugs and sockets
         * are connected to elements that in turn are also fully plugged.
         *
         * \warning Does not perform deadlocking check, i.e. will result in an endless
         * loop if elements are connected to itself through others.
         */
        bool isFullyPlugged() const;

        /**
         * \return A list of all directly connected elements.
         */
        QList<Element*> connectedElements() const;

        /**
         * \return A list of all connected elements (not only directly connected ones)
         * including this element.
         */
        QList<Element*> allConnectedElements() const;

        virtual void paintInterface( QPainter* painter, 
                                     const QStyleOptionGraphicsItem* option, 
                                     const QRect& contentsRect );

        /**
         * Provides the save action.
         */
        QList<QAction*> contextActions();

        /**
         * Returns the static global StorageModel (of the Konduit repository)
         */
        static Konduit::StorageModel *globalStorageModel();

        /**
         * Sets the global StorageModel (for the system repository)
         */
        static void setTestModel( Soprano::Model* );
        
        /**
         * Returns the list of categories used by existing Konduit plugins
         */
        static QStringList listAppletCategories();
                
        /**
         * Returns the list of categories without the "Konduit Elements" prefix
         */
        static QStringList listAppletCategoriesShort();

    public Q_SLOTS:
        /**
         * Shows a default configuration dialog if createStandardGui has been called.
         * Otherwise nothing happens.
         *
         * TODO: We need this slot since Plasma::Applet::showConfigurationInterface
         * is not virtual and depends on KConfigDialog which makes it very very
         * unflexible.
         */
        virtual void showKonduitConfigurationInterface();

    Q_SIGNALS:
        void plugStateChanged();
        void configurationValueChanged( const QString& configName );
        void blackBoxSaved( QString name );
        void openBlackBox( QString name );

        // in any case have a manager with a GUI that wraps elements in groups (or graphs)
        // depending on their connections. Once a group is fully connected it can be started
        // from the GUI (no more start buttons on the sinks).
        // in addition the manager is able to reload configurations (mostly for Plasma)
        // and there should be a checkbox for auto-updating of groups (or whatever we call
        // them). But what does auto-updating mean? every n seconds? When the model changes?
        // maybe sources could have a changed signal? or sockets? or maybe sockets call a method
        // in plug when the data changes?

    protected:
        Element( QGraphicsItem* parent, const QString& serviceId, uint appletId );
        Element( QObject* parent, const QVariantList& args );

        /**
         * Convenience method to access the main
         * Nepomuk data model.
         */
        Soprano::Model* nepomukMainModel() const;

        /**
         * Call this method in the constructor of Elements to initialize
         * the possible configuration options.
         */
        void addConfigurationOption( const ConfigurationOption& option );

        /**
         * Creates a default GUI from the configuration options.
         */
        void createStandardGui();

        /**
         * Handles the update of the connections.
         * Make sure that implementations call the parent method.
         */
        virtual QVariant itemChange( GraphicsItemChange change, const QVariant& value );
        void contextMenuEvent( QGraphicsSceneContextMenuEvent* event );

        /**
         * The recommended size of the applet. Takes into accout the
         * configured plugs and sockets.
         *
         * \sa contentsSizeHint
         */
        virtual QSizeF sizeHint( Qt::SizeHint which, const QSizeF& constraint = QSizeF() ) const;

        /**
         * Size hint for the contents without the Plasma frame.
         */
        virtual QSizeF contentsSizeHint( Qt::SizeHint which, const QSizeF& constraint = QSizeF() ) const;

        /**
         * Mouse Events
         */
        virtual void hoverMoveEvent ( QGraphicsSceneHoverEvent * event );
        virtual void hoverLeaveEvent ( QGraphicsSceneHoverEvent * event );
        virtual void mousePressEvent(QGraphicsSceneMouseEvent *event);
        virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
        virtual void mouseMoveEvent(QGraphicsSceneMouseEvent *event);


    private Q_SLOTS:
        void slotPlugStateChanged( Konduit::Plug* oldPlug, Konduit::Plug* newPlug );
        void slotPlugStateChanged( Konduit::Socket* oldSocket, Konduit::Socket* newSocket );
        void slotSaveGroup();
        void slotSaveBlackBox();
        void slotOpenBlackBox();
        void updateStandardGui();
        void updatePositions();

    private:
        void initialize();
        bool isFullyPlugged( const Element* ) const;
        void moveToGroup( int );

        class Private;
        Private* const d;

    };
    
}

#endif
