/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "fileoutputelement.h"
#include "plug.h"
#include "socket.h"

#include <kio/netaccess.h>
#include <KTemporaryFile>

#include <QtCore/QFile>
#include <QtCore/QTextStream>

#include <Soprano/Serializer>
#include <Soprano/RdfSerialization>
#include <Soprano/PluginManager>
#include <Soprano/StatementIterator>


class Konduit::Elements::FileOutputElement::Private
{
public:
};


Konduit::Elements::FileOutputElement::FileOutputElement( QObject* parent, const QVariantList& args )
    : SinkElement( parent, args ),
      d( 0 )
{
    ConfigurationOption o( "url", QVariant::String );
    o.setLabel( i18n( "Url" ) );
    addConfigurationOption( o );

    ConfigurationOption serializationOption( "serialization", QVariant::String, Soprano::serializationMimeType( Soprano::SerializationNQuads ) );
    serializationOption.setForceSuggestedValue( true );
    QVariantList serializations;
    serializations << Soprano::serializationMimeType( Soprano::SerializationRdfXml )
                   << Soprano::serializationMimeType( Soprano::SerializationTurtle )
                   << Soprano::serializationMimeType( Soprano::SerializationTrig )
                   << "application/n-triples"
                   << Soprano::serializationMimeType( Soprano::SerializationNQuads );
    serializationOption.setSuggestedValues( serializations );
    serializationOption.setLabel( i18n( "Serialization" ) );
    addConfigurationOption( serializationOption );
}


Konduit::Elements::FileOutputElement::~FileOutputElement()
{
}


void Konduit::Elements::FileOutputElement::setUrl( const KUrl& url )
{
    setOption( "url", ConfigurationSetting( url.url() ) );
}


KUrl Konduit::Elements::FileOutputElement::url() const
{
    return optionSetting( "url" ).value().toString();
}


void Konduit::Elements::FileOutputElement::setSerialization( Soprano::RdfSerialization s, const QString& userSerialization )
{
    setOption( "serialization", ConfigurationSetting( Soprano::serializationMimeType( s, userSerialization ) ) );
}


Soprano::RdfSerialization Konduit::Elements::FileOutputElement::serialization() const
{
    QString s = optionSetting( "serialization" ).value().toString();
    return Soprano::mimeTypeToSerialization( s );
}


QString Konduit::Elements::FileOutputElement::userSerialization() const
{
    return optionSetting( "serialization" ).value().toString();
}


bool Konduit::Elements::FileOutputElement::save( const Plug* plug )
{
    if ( !plug->socket() ) {
        return false;
    }

    QString savePath;

    KUrl saveUrl = url();

    if ( saveUrl.isLocalFile() ) {
        savePath = saveUrl.path();
    }
    else {
        KTemporaryFile tmp;
        tmp.setAutoRemove(false);
        tmp.open();
        savePath = tmp.fileName();
    }

    const Soprano::Serializer* serializer = Soprano::PluginManager::instance()->discoverSerializerForSerialization( serialization(), userSerialization() );
    if ( serializer ) {
        QFile file( savePath );
        if ( file.open( QIODevice::WriteOnly ) ) {
            QTextStream stream( &file );
            if ( !serializer->serialize( plug->socket()->data(), stream, serialization(), userSerialization() ) ) {
                file.remove();
                return false;
            }
        }
        else {
            return false;
        }

        if ( !saveUrl.isLocalFile() ) {
            bool success = KIO::NetAccess::upload( savePath, saveUrl, 0 );
            file.remove(); // remove temp file
            return success;
        }

        return true;
    }
    else {
        return false;
    }
}

#include "fileoutputelement.moc"
