/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "konduitstoragemodel.h"
#include "vocabulary.h"
#include "configurationoption.h"
#include "configurationsetting.h"
#include "element.h"
#include "switchelement.h"
#include "inputelement.h"
#include "outputelement.h"
#include "plug.h"
#include "socket.h"
#include "source.h"
#include "sink.h"

#include <Soprano/Global>
#include <Soprano/Model>
#include <Soprano/Node>
#include <Soprano/LiteralValue>
#include <Soprano/StatementIterator>
#include <Soprano/QueryResultIterator>
#include <Soprano/Vocabulary/RDF>
#include <Soprano/Vocabulary/RDFS>
#include <Soprano/Vocabulary/XMLSchema>
#include <Soprano/PluginManager>
#include <Soprano/Statement>
#include <Soprano/Serializer>
#include <Soprano/Parser>

#include <KRandom>
#include <KGlobal>
#include <KStandardDirs>
#include <KInputDialog>
#include <KServiceTypeTrader>
#include <KSaveFile>

#include <plasma/applet.h>

#include <QMessageBox>
#include <QTextStream>
#include <QFile>
#include <QTextStream>


using namespace Soprano;

namespace {
    QUrl createUniqueUri( Soprano::Model* model ) {
        QUrl s;
        while( 1 ) {
            // Should we use the Nepomuk localhost whatever namespace here?
            s = "http://konduit.org/konduits#" + KRandom::randomString( 20 );
            if( !model->containsContext( s ) &&
                !model->containsAnyStatement( Soprano::Statement( s, Soprano::Node(), Soprano::Node() ) ) &&
                !model->containsAnyStatement( Soprano::Statement( Soprano::Node(), s, Soprano::Node() ) ) &&
                !model->containsAnyStatement( Soprano::Statement( Soprano::Node(), Soprano::Node(), s ) ) )
                return s;
        }
    }

    Konduit::Element* createElement( const QString& type ) {
        QVariantList args;
        int id = 0;
        return ( Konduit::Element* )Plasma::Applet::load( type, id, args );
    }

    QList<Soprano::Node> createConfigNodes( const Konduit::ConfigurationOption& option, const Konduit::ConfigurationSetting& setting ) {
        Q_UNUSED( option );
        QList<Soprano::Node> literals;
        QList<QVariant> values = setting.values();
        foreach( QVariant v, values ) {
            if ( v.type() == QVariant::Url ) {
                literals.append( Soprano::Node( v.toUrl() ) );
            }
            else {
                // TODO: check if the type is supported at all
                literals.append( Soprano::LiteralValue( v ) );
            }
        }
        return literals;
    }

    QRectF decodeGeometry( const QString& s ) {
        QStringList sl = s.split( ',' );
        if ( sl.count() == 4 ) {
            return QRectF( sl[0].toDouble(), sl[1].toDouble(), sl[2].toDouble(), sl[3].toDouble() );
        }
        else {
            return QRectF();
        }
    }
}


Konduit::StorageModel::StorageModel()
    : Soprano::FilterModel()
{
    QString storageDir = KGlobal::dirs()->locateLocal( "data", "konduit/storage/" );
    Q_ASSERT( !storageDir.isEmpty() );
    m_sopranoModel = Soprano::createModel( Soprano::BackendSettings() << Soprano::BackendSetting( Soprano::BackendOptionStorageDir, storageDir ) );
    Q_ASSERT( m_sopranoModel );
    setParentModel( m_sopranoModel );
}


Konduit::StorageModel::~StorageModel()
{
    delete m_sopranoModel;
}


QUrl Konduit::StorageModel::findKonduit( const QString& name ) const
{
    return findKonduitMain(name, false);
}

QUrl Konduit::StorageModel::findKonduitBlackBox( const QString& name ) const
{
    return findKonduitMain(name, true);
}

QUrl Konduit::StorageModel::findKonduitMain( const QString& name, bool isBlackBox ) const 
{
        Soprano::QueryResultIterator it = executeQuery( QString( "select ?r where { "
                                                             "?r a <%1> . "
                                                             "?r <%2> \"%3\"^^<%4> . }" )
                                                    .arg( isBlackBox ?
                                                          Konduit::Vocabulary::KonduitBlackBox().toString() :
                                                          Konduit::Vocabulary::Konduit().toString() )
                                                    .arg( Soprano::Vocabulary::RDFS::label().toString() )
                                                    .arg( name )
                                                    .arg( Soprano::Vocabulary::XMLSchema::string().toString() ),
                                                    Soprano::Query::QueryLanguageSparql );
    if ( it.next() ) {
        return it.binding( "r" ).uri();
    }
    else {
        kDebug() << "findKonduitMain: " << name << " not found in the repository.";
        return QUrl();
    }
}

QUrl Konduit::StorageModel::saveKonduit( Element* someElement, const QString& name )
{
    return saveKonduitMain(someElement, name, false);
}

QUrl Konduit::StorageModel::saveKonduitBlackBox( Element* someElement, const QString& name, const QString& comment, const QString& category )
{
    //check that the name isn't that of a previously saved Konduit
    foreach ( const QString& category, Konduit::Element::listAppletCategories() ) {
        foreach( const KPluginInfo& kpi, Plasma::Applet::listAppletInfo(category) ) {
            if( kpi.pluginName() == "konduit_" + name) {
                return QUrl();//if it exists, return an empty name
            }
        }
    }

    QUrl s = saveKonduitMain(someElement, name, true);

    if(!s.isEmpty()) {//if blackbox saved, write desktop file
        QString filePath = pluginPath( name );
        kDebug() << "Saving blackbox desktop file to " + filePath;
        if( saveDesktopFile( filePath, name, comment, category ) ) {
            return s;
        }
        //otherwise - if .desktop file could not be written
        deleteBlackBox( name );
        kDebug() << "Error: desktop file could not be saved!";
        return QUrl();
    }
    return s;
}

bool Konduit::StorageModel::saveDesktopFile( const QString& filePath, const QString& name, const QString& comment, const QString& category )
{
    KSaveFile bbFile(filePath);

    if (bbFile.open(QIODevice::WriteOnly | QIODevice::Text)) {
        QTextStream out(&bbFile);
        out << "[Desktop Entry]" <<
        "\nEncoding=UTF-8" <<
        "\nName=" << name <<
        "\nComment=" << comment << 
        "\nType=Service" <<
        "\nServiceTypes=Plasma/Applet" <<
        "\nIcon=konduit-input" <<
        "\n" <<
        "\nX-KDE-Library=konduit_blackboxloader" <<
        "\nX-KDE-PluginInfo-Author=" <<
        "\nX-KDE-PluginInfo-Email=" <<
        "\nX-KDE-PluginInfo-Name=" << name <<
        "\nX-KDE-PluginInfo-Version=0.1" <<
        "\nX-KDE-PluginInfo-Website=http://plasma.kde.org/" <<
        "\nX-KDE-PluginInfo-Category=" << (category.isEmpty() || !category.contains("Konduit Elements -") ? "Konduit Elements - Blackbox" : category) <<
        "\nX-KDE-PluginInfo-Depends=" <<
        "\nX-KDE-PluginInfo-License=GPL" <<
        "\nX-KDE-PluginInfo-EnabledByDefault=true " << endl;

        bbFile.finalize();
        bbFile.close();
        
        return true;
    }
    
    return false;
}

bool Konduit::StorageModel::validateBlackBox( Element* someElement )
{
        bool ok = true;//proper black box
        QList<QString> inputNames;
        QList<QString> outputNames;

        if(someElement->allConnectedElements().count() < 1) 
        { 
            ok = false; //no elements connected to it
            kDebug() << "No connected elements!";
        }

        foreach( Element* element, someElement->allConnectedElements() ) {//check that the konduit has all connections properly set
            if(element->pluginName() == "konduit_input") {
                Konduit::Elements::InputElement* inpE = (Konduit::Elements::InputElement*) element;

                if(!inpE->socket()->plug() || inpE->plug()->socket()) {//correct if: socket plugged in , plug isn't
                    ok = false;
                    kDebug() << "Input element connectivity problem: " << inpE->getName();
                }

                if( inputNames.contains( inpE->getName() ) ) {
                    ok = false;//found a duplicate name
                    kDebug() << "Duplicate input element name: " << inpE->getName();
                }
                else {
                    inputNames.append( inpE->getName() );
                    //kDebug() << "inserted " << inpE->getName();
                }
            }
            else if (element->pluginName() == "konduit_output") {
                Konduit::Elements::OutputElement* outE = (Konduit::Elements::OutputElement*) element;

                if(!outE->plug()->socket() || outE->socket()->plug()) {//correct if: socket unplugged, plug plugged
                    ok = false;
                    kDebug() << "Output element connectivity problem: " << outE->getName();
                }

                if( outputNames.contains( outE->getName() ) ) {
                    ok = false;//found a duplicate name
                    kDebug() << "Duplicate output element name: " << outE->getName();
                }
                else {
                    outputNames.append( outE->getName() );
                    //kDebug() << "inserted " << outE->getName();
                }
            }
            else { //all other elements need to have all plugs and sockets connected
                if ( element->isSink() ) { //a Sink has Plugs
                    QList<Plug*> plugs = dynamic_cast<Sink*>( element )->allPlugs();
                    foreach(Plug* plug, plugs) {
                        if (!plug->socket()) {
                            ok = false;
                            kDebug() << "A plug is disconnected!";
                        }
                    }
                }
                if ( element->isSource() ) { //a Source has Sockets
                    QList<Socket*> sockets = dynamic_cast<Source*>( element )->allSockets();
                    foreach(Socket* socket, sockets) {
                        if (!socket->plug()) {
                            ok = false;
                            kDebug() << "A socket is disconnected!";
                        }
                    }
                }
            }
        }
    
    return ok;
}

QUrl Konduit::StorageModel::saveKonduitMain( Element* someElement, const QString& name, bool isBlackBox )
{
    // FIXME: make sure the Nepomuk repo is running or fallback to storing to memory model + serialization to file

    removeKonduit( name, isBlackBox );

    QUrl konduitGraphUri = createUniqueUri( this );

    // add metadata
    addStatement( konduitGraphUri,
                  Soprano::Vocabulary::RDF::type(),
                  isBlackBox ?
          Konduit::Vocabulary::KonduitBlackBox() : 
          Konduit::Vocabulary::Konduit(),
                  konduitGraphUri );
    addStatement( konduitGraphUri,
                  Soprano::Vocabulary::RDFS::label(),
                  Soprano::LiteralValue( name ),
                  konduitGraphUri );


    // collect elements
    QHash<Element*, QUrl> elements;
    foreach( Element* element, someElement->allConnectedElements() ) {
        elements.insert( element, createUniqueUri( this ) );
    }

    // save it all
    for ( QHash<Element*, QUrl>::const_iterator it = elements.constBegin();
          it != elements.constEnd(); ++it ) {

        Element* element = it.key();
        QUrl elementUri = it.value();

        // save the element
        addStatement( elementUri,
                      Soprano::Vocabulary::RDF::type(),
                      Konduit::Vocabulary::Element(),
                      konduitGraphUri );
        addStatement( elementUri,
                      Konduit::Vocabulary::elementType(),
                      Soprano::LiteralValue( element->pluginName() ),
                      konduitGraphUri );

        // save the geometry
        QRectF geo = element->geometry();
        QString geoString = QString( "%1,%2,%3,%4" )
                            .arg( geo.x() )
                            .arg( geo.y() )
                            .arg( geo.width() )
                            .arg( geo.height() );
        addStatement( elementUri,
                      Konduit::Vocabulary::geometry(),
                      LiteralValue( geoString ),
                      konduitGraphUri );

        // save the configuration
        QList<ConfigurationOption> configurationOptions = element->configurationOptions();
        foreach( ConfigurationOption option, configurationOptions ) {
            QUrl settingUri = createUniqueUri( this );

            // save the option's metadata
            addStatement( settingUri,
                          Soprano::Vocabulary::RDF::type(),
                          Konduit::Vocabulary::ConfigurationSetting(),
                          konduitGraphUri );
            addStatement( settingUri,
                          Soprano::Vocabulary::RDFS::label(),
                          LiteralValue( option.name() ),
                          konduitGraphUri );
            addStatement( settingUri,
                          Konduit::Vocabulary::parentElement(),
                          elementUri,
                          konduitGraphUri );

            // save the values
            QList<Node> configNodes = createConfigNodes( option, element->optionSetting( option.name() ) );
            foreach( Node node, configNodes ) {
                addStatement( settingUri,
                              Konduit::Vocabulary::configurationValue(),
                              node,
                              konduitGraphUri );
            }
        }

        // save the connections
        if ( element->isSink() ) {
            QList<Plug*> plugs = dynamic_cast<Sink*>( element )->allPlugs();
            Q_FOREACH( Plug* plug, plugs ) {
                if ( plug->socket() ) {
                    Element* pluggedSource = dynamic_cast<Element*>( plug->socket()->source() );
                    QString plugName = plug->name();
                    QString socketName = plug->socket()->name();
                    QUrl plugUri = createUniqueUri( this );
                    QUrl socketUri = createUniqueUri( this );

                    // store the plug
                    addStatement( plugUri,
                                  Soprano::Vocabulary::RDF::type(),
                                  Konduit::Vocabulary::Plug(),
                                  konduitGraphUri );
                    addStatement( plugUri,
                                  Konduit::Vocabulary::parentElement(),
                                  elementUri,
                                  konduitGraphUri );
                    addStatement( plugUri,
                                  Soprano::Vocabulary::RDFS::label(),
                                  Soprano::LiteralValue( plugName ),
                                  konduitGraphUri );

                    // store the socket
                    addStatement( socketUri,
                                  Soprano::Vocabulary::RDF::type(),
                                  Konduit::Vocabulary::Socket(),
                                  konduitGraphUri );
                    addStatement( socketUri,
                                  Konduit::Vocabulary::parentElement(),
                                  elements[pluggedSource],
                                  konduitGraphUri );
                    addStatement( socketUri,
                                  Soprano::Vocabulary::RDFS::label(),
                                  Soprano::LiteralValue( socketName ),
                                  konduitGraphUri );

                    // store the connection
                    addStatement( plugUri,
                                  Konduit::Vocabulary::pluggedIn(),
                                  socketUri,
                                  konduitGraphUri );
                }
            }
        }
    }

    return konduitGraphUri;
}

bool Konduit::StorageModel::removeKonduit( const QString& name, bool isBlackBox )
{
    QUrl uri;
    if(isBlackBox) {
        uri = findKonduitBlackBox( name );
    }
    else {
        uri = findKonduit( name );
    }

    if ( uri.isValid() ) {
        removeContext( uri );
        return true;
    }
    
    kDebug() << "Error: Cannot remove! Invalid URI.";
    return false;
}


Konduit::Element* Konduit::StorageModel::loadKonduit( const QString& name )
{
    return loadKonduitMain(name, false);
}


Konduit::Element* Konduit::StorageModel::loadKonduitBlackBox( const QString& name )
{
    return loadKonduitMain(name, true);
}


Konduit::Element* Konduit::StorageModel::loadKonduitMain( const QString& name, bool isBlackBox  )
{
    QUrl uri = findKonduitMain( name, isBlackBox );
    if ( uri.isValid() ) {
        // load elements
        QHash<QUrl, Element*> elements;
        Soprano::QueryResultIterator it = executeQuery( QString( "select ?r ?type ?geo where { "
                                                                 "graph <%1> { "
                                                                 "?r a <%3> . "
                                                                 "?r <%4> ?type . "
                                                                 "OPTIONAL { ?r <%5> ?geo } . } . }" )
                                                        .arg( uri.toString() )
                                                        .arg( Konduit::Vocabulary::Element().toString() )
                                                        .arg( Konduit::Vocabulary::elementType().toString() )
                                                        .arg( Konduit::Vocabulary::geometry().toString() ),
                                                        Soprano::Query::QueryLanguageSparql );
        while ( it.next() ) {
            Element* element = createElement( it.binding( "type" ).toString() );
            elements.insert( it.binding( "r" ).uri(), element );

            // load configuration
            Soprano::QueryResultIterator configSettingUriIt = executeQuery( QString( "select distinct ?configUri ?configName where { "
                                                                                     "graph <%1> { "
                                                                                     "?configUri a <%2> . "
                                                                                     "?configUri <%3> ?configName . "
                                                                                     "?configUri <%4> <%5> . } . }" )
                                                                            .arg( uri.toString() )
                                                                            .arg( Konduit::Vocabulary::ConfigurationSetting().toString() )
                                                                            .arg( Soprano::Vocabulary::RDFS::label().toString() )
                                                                            .arg( Konduit::Vocabulary::parentElement().toString() )
                                                                            .arg( it.binding( "r" ).toString() ),
                                                                            Soprano::Query::QueryLanguageSparql );
                                                                            
            // load geometry first, so items will get updated when setting the options
            QRectF geo = decodeGeometry( it.binding( "geo" ).toString() );
            if ( geo.isValid() ) {
                element->setGeometry( geo );
            }
                                                                            
            while ( configSettingUriIt.next() ) {
                // load values
                Soprano::QueryResultIterator configValuesIt = executeQuery( QString( "select ?v where { "
                                                                                     "graph <%1> { "
                                                                                     "<%2> <%3> ?v . } . }" )
                                                                            .arg( uri.toString() )
                                                                            .arg( configSettingUriIt.binding( "configUri" ).toString() )
                                                                            .arg( Konduit::Vocabulary::configurationValue().toString() ),
                                                                            Query::QueryLanguageSparql );
                Konduit::ConfigurationSetting setting;
                while ( configValuesIt.next() ) {
                    Soprano::Node value = configValuesIt.binding( "v" );
                    if ( value.isLiteral() ) {
                        setting.addValue( value.literal().variant() );
                    }
                    else {
                        setting.addValue( value.uri() );
                    }
                }
                element->setOption( configSettingUriIt.binding( "configName" ).toString(), setting );
            }

        }

        // loads connections
        it = executeQuery( QString( "select ?r1 ?r2 ?plugName ?socketName where { "
                                    "graph <%1> { "
                                    "?plug a <%2> . "
                                    "?plug <%3> ?plugName . "
                                    "?plug <%4> ?r1 . "
                                    "?plug <%5> ?socket . "
                                    "?socket a <%6> . "
                                    "?socket <%7> ?r2 . "
                                    "?socket <%3> ?socketName . } . }" )
                           .arg( uri.toString() )
                           .arg( Konduit::Vocabulary::Plug().toString() )
                           .arg( Soprano::Vocabulary::RDFS::label().toString() )
                           .arg( Konduit::Vocabulary::parentElement().toString() )
                           .arg( Konduit::Vocabulary::pluggedIn().toString() )
                           .arg( Konduit::Vocabulary::Socket().toString() )
                           .arg( Konduit::Vocabulary::parentElement().toString() ),
                           Soprano::Query::QueryLanguageSparql );

        while ( it.next() ) {
            Sink* sinkElement = dynamic_cast<Sink*>( elements[it.binding( "r1" ).uri()] );
            Source* sourceElement = dynamic_cast<Source*>( elements[it.binding( "r2" ).uri()] );
            QString plugName = it.binding( "plugName" ).toString();
            QString socketName = it.binding( "socketName" ).toString();

            // FIXME: check if this is proper data
            sinkElement->plug( plugName )->plugIn( sourceElement->socket( socketName ) );
        }

        if(elements.values().isEmpty())  {
            return 0;
        }
        else {
            return elements.values().first();
        }
    }
    else {
        return 0;
    }
}

bool Konduit::StorageModel::deleteBlackBox( const QString& name )
{
    if( removeKonduit( name, true ) ) {
        kDebug() << "Removed all statements of " << name;
        
        QString filePath = pluginPath( name );
        QFile file(filePath);;
        if( file.exists() ) {
            kDebug() << "Removing desktop file " + filePath;
            return file.remove();
        }
        kDebug() << "Desktop file doesn't exist.";
        return false;
    }
    
    kDebug() << "Could not remove Konduit from repository! Desktop file was not removed.";
    return false;
}

const QString kbbFileSeparator = "[Konduit Blackbox Serialization]";

bool Konduit::StorageModel::exportBlackBox( const QString& name, const QString& filePath )
{
    QUrl uri = findKonduitBlackBox( name );
    if( uri.isEmpty() ) {//if id does not exist
        return false;
    }

    kDebug() << "Exporting [" << name << "] to: " << filePath;

    //FIXME: use KSaveFile file(filePath)
    QFile file(filePath);
    file.remove();//overwrite, if it exists

    QFile origFile( pluginPath( name ) );
    if( !origFile.copy( filePath ) ) { //if .desktop file creation is unsuccessful
        kDebug() << "Can not copy .desktop file";
        return false;
    }
    
    
    Soprano::StatementIterator it = listStatementsInContext( uri ) ;
    Soprano::Model *mm = Soprano::createModel();
    mm->addStatements(it.allStatements());
    const Soprano::Serializer *s = 
        Soprano::PluginManager::instance()->discoverSerializerForSerialization( Soprano::SerializationNQuads);
    
    if( file.exists() ) {
        if (file.open(QIODevice::Append | QFile::Text))
        {
            QTextStream stream(&file);
            stream << kbbFileSeparator << endl;
            s->serialize(mm->listStatements(), stream, Soprano::SerializationNQuads);
            file.close();
            return true;
        }
        kDebug() << "Can not append RDF serialiazion to .desktop file!";
        return false;
    }
    
    kDebug() << "Desktop file doesn't exist!";
    return false;
}

bool Konduit::StorageModel::importBlackBox( const QString& filePath )
{
    kDebug() << "Importing from: "  << filePath;
    
    QString de = QRegExp::escape( "[Desktop Entry]" );
    QString kbbfs = QRegExp::escape( kbbFileSeparator); 
    QString regExp = "(" + de + ")(.*)(" + kbbfs + ")(.*)";
    
    QRegExp rx( regExp );
    
    QFile file(filePath);
    if( file.exists() ) {
        if ( file.open( QIODevice::ReadOnly | QIODevice::Text ) ) {
            QTextStream in( &file );
            QString all = in.readAll();
            
            if ( rx.indexIn( all ) != -1 ) { //if both entries (desktop + rdf) found in file
                QString desktopFileContents = rx.cap(2);
                QString konduitStatementsContents = rx.cap(4);
                
                //remove everything before first URI
                int firstUriPos = konduitStatementsContents.indexOf( "<" );
                konduitStatementsContents.remove( 0, firstUriPos );

                QString pluginName;
                QRegExp rx2( "(Name=)(.*)(\nComment=)" );
                if (rx2.indexIn( desktopFileContents ) != -1) {
                    pluginName = rx2.cap(2);
                }
                else {
                    pluginName = "untitled" + KRandom::randomString( 5 );
                }
                
                QString savePath = pluginPath( pluginName );
                KSaveFile bbFile( savePath );
                
                if (bbFile.open(QIODevice::WriteOnly | QIODevice::Text)) {
                    //write .desktop file
                    QTextStream out(&bbFile);
                    out << "[Desktop Entry]" << desktopFileContents;
                    bbFile.finalize();
                    bbFile.close();

                    QList<Soprano::Statement> all;
                    const Soprano::Parser* p = 
                        Soprano::PluginManager::instance()->discoverParserForSerialization( Soprano::SerializationNQuads );
                    if ( p )
                    {
                        all = p->parseString( konduitStatementsContents, QString(), Soprano::SerializationNQuads ).allStatements();
                    }
                    
                    if ( all.isEmpty() ) {//if there was a problem parsing the input
                        kDebug() << "No statements found.";
                        QFile bbF( savePath );
                        if( bbF.remove() ) {
                            kDebug() << "Error. Removed " << savePath;
                        }
                        else {
                            kDebug() << "Error. But DID NOT remove " <<  savePath;
                        }
                        return false;
                    }
                    else {
                        addStatements(all);
                        return true;
                    }

                }
            }//end parsing contents
            kDebug() << "Error parsing file contents!";
            return false;
        }//end processing OPEN file
        kDebug() << "Error opening file!";
        return false;
    }//end processing file
    
    kDebug() << "File doesn't exist!";
    return false;
}

QStringList Konduit::StorageModel::allSavedKonduits()
{
    return allSavedKonduitsMain(false);
}

QStringList Konduit::StorageModel::allSavedKonduitBlackBoxes()
{
    return allSavedKonduitsMain(true);
}

QStringList Konduit::StorageModel::allSavedKonduitsMain(bool isBlackBox) {
    QStringList sl;
    Soprano::QueryResultIterator it = executeQuery( QString( "select ?label where { "
                                                             "?r a <%1> . "
                                                             "?r <%2> ?label . }" )
                                                    .arg(   isBlackBox ?
                                Konduit::Vocabulary::KonduitBlackBox().toString() : 
                                        Konduit::Vocabulary::Konduit().toString() )
                                                    .arg( Soprano::Vocabulary::RDFS::label().toString() ),
                                                    Soprano::Query::QueryLanguageSparql );
    while ( it.next() ) {
        sl << it.binding( "label" ).toString();
    }
    return sl;

}

QString Konduit::StorageModel::pluginPath( const QString& pluginName )
{
    QString saveName = "konduit_" + pluginName + ".desktop";
    QString savePath = KGlobal::dirs()->saveLocation("services", "");
    savePath = savePath + saveName;
    return savePath;
}

QString Konduit::StorageModel::findBlackboxFromFile( const QString& filePath ) {
    QFile file( filePath );
    if ( file.open( QIODevice::ReadOnly | QIODevice::Text ) ) {//if file open
        QTextStream in( &file );
        QString all = in.readAll();
        
        QString pluginName;
        QRegExp rx2( "(Name=)(.*)(\nComment=)" );
        if (rx2.indexIn( all ) != -1) {//if name found
            pluginName = rx2.cap( 2 );
            if ( !findKonduitBlackBox( pluginName ).isEmpty() ) {//if it is in the repository
                return pluginName;
            }
        }
        file.close();
        kDebug() << "Can not find Name entry!";
        return QString();
    }
    
    kDebug() << "Can not open file!";
    return QString();
}
