/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "plug.h"
#include "socket.h"
#include "sink.h"
#include "cord.h"

#include <QtGui/QPainter>
#include <QtGui/QGraphicsScene>
#include <QtCore/QPointer>

#include <Soprano/StatementIterator>

#include <KDebug>


class Konduit::Plug::Private
{
public:
    Private( Plug* plug )
        : parentSink( 0 ),
          socket( 0 ),
          boundingRect( 0, 0, 10, 20 ),
          m_plug( plug ) {
    }

    Sink* parentSink;
    QPointer<Socket> socket;
    QString name;

    QRectF boundingRect;

    QPointer<Cord> cord;

private:
    Plug* m_plug;
};


Konduit::Plug::Plug( Sink* sink, const QString& name )
    : QObject( 0 ),
      QGraphicsItem( dynamic_cast<QGraphicsItem*>( sink )->parentItem() ),
      d( new Private(this) )
{
    d->parentSink = sink;
    d->name = name;
    d->cord = new Cord( this, parentItem() );
    setZValue( dynamic_cast<QGraphicsItem*>( sink )->zValue()+1 );
    setToolTip( name );
}


Konduit::Plug::~Plug()
{
    unplug();
    delete d->cord;
    delete d;
}


QString Konduit::Plug::name() const
{
    return d->name;
}


Konduit::Sink* Konduit::Plug::sink() const
{
    return d->parentSink;
}


bool Konduit::Plug::plugIn( Socket* socket )
{
    if ( d->socket != socket ) {
        Socket* oldSock = d->socket;
        unplug();
        d->socket = socket;
        d->socket->plugIn( this );
        d->cord->reset();

        emit plugStateChanged( oldSock, socket );
    }

    return true;
}


void Konduit::Plug::unplug()
{
    if ( d->socket ) {
        Socket* s = d->socket;
        d->socket = 0;
        s->unplug();
        d->cord->reset();

        emit plugStateChanged( s, 0 );
    }
}


Konduit::Socket* Konduit::Plug::socket() const
{
    return d->socket;
}


QRectF Konduit::Plug::boundingRect() const
{
    return d->boundingRect;
}


void Konduit::Plug::paint( QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget )
{
    Q_UNUSED( option );
    Q_UNUSED( widget );
    painter->fillRect( 0, 0, 10, 20, Qt::white );
    painter->drawLine( 0, 0, 10, 0 );
    painter->drawLine( 10, 0, 10, 20 );
    painter->drawLine( 0, 20, 10, 20 );
}


void Konduit::Plug::updatePosition()
{
    d->cord->reset();
}


QVariant Konduit::Plug::itemChange( GraphicsItemChange change, const QVariant& value )
{
    if ( change == QGraphicsItem::ItemPositionHasChanged ) {
        d->cord->reset();
    }
    else if ( change == QGraphicsItem::ItemSceneChange ) {
        kDebug() << "ItemSceneChange";
        QGraphicsScene* newScene = value.value<QGraphicsScene*>();
        if ( scene() ) {
            kDebug() << "removing cord from scene" << scene();
            scene()->removeItem( d->cord );
        }
        if ( newScene ) {
            kDebug() << "adding cord to scene" << newScene;
            newScene->addItem( d->cord );
        }
    }

    return QGraphicsItem::itemChange( change, value );
}


Soprano::StatementIterator Konduit::Plug::data() const
{
    if ( socket() ) {
        return socket()->data();
    }
    else {
        return 0;
    }
}

#include "plug.moc"
