/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "datatableelement.h"
#include "plug.h"
#include "source.h"
#include "socket.h"

#include <Soprano/Soprano>
#include <Soprano/StatementIterator>
#include <Soprano/Model>
#include <Soprano/QueryResultIterator>
#include <Soprano/Node>
#include <Soprano/NodeIterator>
#include <Soprano/Query/QueryLanguage>

#include <QtCore/QTextStream>
#include <QtGui/QTextBrowser>
#include <QtGlobal>
#include <QGraphicsLinearLayout>

#include <plasma/widgets/label.h>
#include <plasma/widgets/pushbutton.h>


class Konduit::Elements::DataTableElement::Private
{
public:
    Plasma::Label* view;

    void init( DataTableElement* e ) {
        e->addPlug( "input" );

        QGraphicsLinearLayout* mainLay = new QGraphicsLinearLayout( Qt::Vertical, e);

        Plasma::PushButton* startButton = new Plasma::PushButton( e );
        startButton->setText( "Start" );
        mainLay->addItem( startButton );
        connect( startButton, SIGNAL( clicked() ), e, SLOT( start() ) );
    }
};


Konduit::Elements::DataTableElement::DataTableElement( QGraphicsItem* parent, const QString& serviceId, uint appletId )
    : SinkElement( parent, serviceId, appletId ),
      d( new Private() )
{
    d->init( this );
}


Konduit::Elements::DataTableElement::DataTableElement( QObject* parent, const QVariantList& args )
    : SinkElement( parent, args ),
      d( new Private() )
{
    d->init( this );
    setToolTip("Data Table Dump");
    resize( QSizeF(130, 80) );
    setMinimumSize( QSizeF(90, 60) );
}


Konduit::Elements::DataTableElement::~DataTableElement()
{
    delete d;
}

bool Konduit::Elements::DataTableElement::start()
{
    //kDebug();
    // get the data from the source and dump it
    Plug* p = plug( "input" );
    if ( p->socket() ) {
        Soprano::StatementIterator it = p->socket()->data();
        QList<Soprano::Statement> allStatements = it.allElements();

        displaySeparateTables(allStatements);
        
    it.close();
        return true;
    }
    else {
        return false;
    }
}


void Konduit::Elements::DataTableElement::displaySeparateTables(QList<Soprano::Statement> allStatements)
{
    int i,j;
    QMap<QString, QList<QList <QString> > > tables;//(subject,table) map
    QMap<QString, QString> types; //(subject,type) MAP
    QList<QList <QString> > table;

    QList<QString> subjects;
    QList<QString> predicates;
    QList<QString> objects;
    //output all statements
    foreach (Soprano::Statement s, allStatements ) {
            subjects << s.subject().toString();
            predicates << s.predicate().toString();
            objects << s.object().toString();    
        }
        
    //step 1: for each type: create tables and (subject,type) associations
    for(i=0; i<predicates.count(); i++) {
        if(predicates[i] == "http://www.w3.org/1999/02/22-rdf-syntax-ns#type") {
        tables.insert(objects[i], *(new QList<QList <QString> >())); 
        types.insert(subjects[i], objects[i]); 
        }
    }

    //step 1b: add types for subjects, that did not have one
    //FIXME: find correct type through a Soprano query, and create separate table for each one
    for(i=0; i<predicates.count(); i++) {
        QString subj = subjects[i];
        if( ! types.contains( subj ) ) {
            tables.insert( "<undefined-type>", *(new QList<QList <QString> >()) ); 
            types.insert( subj, "<undefined-type>" ); 
        }
    }
        
        
    //step 2: construct table headers
    foreach(QString s, tables.keys()) {
        tables[s].append(*(new QList<QString>()));
        tables[s][0].append("Resource ID");
    }
    for(i=0; i<predicates.count(); i++) {
        QString subj = subjects[i];
        QString pred = predicates[i];
        QString type = types[subj];
        
        if(tables[type][0].lastIndexOf(pred) < 0)
            tables[type][0].append(pred);
    }
    
    //step 3: fill tables
    for(i=0; i<subjects.count(); i++) {
        QString subj = subjects[i];
        QString pred = predicates[i];
        QString obj = objects[i];
        QString type = types[subj];
        int r, c;
        
        if( (r = onColumn(subj, 0, tables[type])) < 0) {//if subject not yet added
            //init row
            tables[type].append(*(new QList<QString>()));
            int max = tables[type].count()-1;
            for(j=0; j<tables[type][0].count(); j++) { tables[type][max].append("");}
            
            tables[type][max][0] = subj;
            r = max;
        }
        c = tables[type][0].lastIndexOf(pred);
        tables[type][r][c] = obj;
    }
    
    //step 4: output tables in HTML
    foreach(QString typeS, tables.uniqueKeys()) {
        QString buf;
        QTextStream bufS( &buf );    
        bufS << "<table border=\"1\">" << endl;
        bufS << "<tr>" << endl;
        for(j=0; j<tables[typeS][0].count(); j++) {
            bufS << "<th> "; 
            bufS << getRDFLabel(tables[typeS][0][j]);
            bufS << "</th>" << endl;
        }
        bufS << "</tr>" << endl;
        for(i=1; i<tables[typeS].count(); i++) {
            bufS << "<tr>" << endl;
            for(j=0; j<tables[typeS][i].count(); j++) {
                bufS << "<td> "; 
                bufS << tables[typeS][i][j];
                bufS << "</td>" << endl;
            }
            bufS << "</tr>" << endl;
    
        }
        bufS << "</table>" << endl;
        
        QTextBrowser* b = new QTextBrowser(0);
        b->setHtml(buf);
        b->show();            
    }
}

    
int Konduit::Elements::DataTableElement::onColumn(QString s, int c, QList<QList <QString> > t) {
        int i;
    if(t.count() < 0 || t[0].count() < c)//if table empty or column too big
        return -1;
    for(i=0; i<t.count(); i++)
    {
        if(t[i][c] == s)
            return i;
    } 
    return -1;
}

QString Konduit::Elements::DataTableElement::getRDFLabel(QString resource) {
    if (resource.section("#", 1, 1) == "")
        return resource;
    else {
        //FIXME: retrieve label with a query through Soprano
        return resource.section("#", 1, 1);
    }
}

#include "datatableelement.moc"
