/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Sebastian Trueg <trueg@kde.org>
 *               2000 Espen Sand, espen@kde.org
 *               2001-2002 Marc Mutz <mutz@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "identitykcm.h"
#include "identitylistview.h"
#include "identitydialog.h"
#include "newidentitydialog.h"

#include <kpimidentities/identity.h>

#include <QBoxLayout>
#include <QCheckBox>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QHideEvent>
#include <QLabel>
#include <QLayout>
#include <QList>
#include <QRadioButton>
#include <QLineEdit>
#include <QGroupBox>
#include <QToolTip>
#include <QListWidget>
#include <QValidator>
#include <QVBoxLayout>
#include <QWhatsThis>
#include <QDBusConnection>
#include <q3buttongroup.h>
#include <QTextCodec>
#include <q3header.h>
#include <QMenu>

#include <klocale.h>
#include <kcharsets.h>
#include <kascii.h>
#include <knuminput.h>
#include <kfontchooser.h>
#include <kmessagebox.h>
#include <kurlrequester.h>
#include <kseparator.h>
#include <kiconloader.h>
#include <kstandarddirs.h>
#include <kwindowsystem.h>
#include <kconfig.h>
#include <kcmultidialog.h>
#include <knotifyconfigwidget.h>
#include <k3resolver.h>
#include <kconfiggroup.h>
#include <kbuttongroup.h>
#include <kcolorcombo.h>
#include <kfontrequester.h>
#include <kicondialog.h>
#include <kkeysequencewidget.h>
#include <KIconButton>
#include <KRandom>
#include <KColorScheme>
#include <KDebug>

#include <kpimidentities/identitymanager.h>
#include <kpimidentities/identity.h>

using namespace KMail;

K_PLUGIN_FACTORY( KCMIdentityFactory, registerPlugin<KCMIdentity>(); )
K_EXPORT_PLUGIN( KCMIdentityFactory("kcm_identity") )


static KPIMIdentities::IdentityManager* s_identityManager = 0;

KPIMIdentities::IdentityManager* KCMIdentity::identityManager()
{
    return s_identityManager;
}


KCMIdentity::KCMIdentity( QWidget* parent, const QVariantList& args )
    : KCModule( KCMIdentityFactory::componentData(), parent, args ),
      m_identityManager( new KPIMIdentities::IdentityManager() ),
      mIdentityDialog( 0 )
{
    s_identityManager = m_identityManager;

    QHBoxLayout *hlay = new QHBoxLayout( this );
    hlay->setSpacing( KDialog::spacingHint() );
    hlay->setMargin( 0 );

    mIdentityList = new IdentityListView( this );
    connect( mIdentityList, SIGNAL( itemSelectionChanged() ),
             SLOT( slotIdentitySelectionChanged() ) );
    connect( this, SIGNAL( changed(bool) ),
             SLOT( slotIdentitySelectionChanged() ) );
    connect( mIdentityList, SIGNAL( rename( KMail::IdentityListViewItem *, const QString & ) ),
             SLOT( slotRenameIdentity(KMail::IdentityListViewItem *, const QString & ) ) );
    connect( mIdentityList, SIGNAL( itemDoubleClicked( QTreeWidgetItem*, int ) ),
             SLOT( slotModifyIdentity() ) );
    connect( mIdentityList, SIGNAL( contextMenu( KMail::IdentityListViewItem *, const QPoint & ) ),
             SLOT( slotContextMenu( KMail::IdentityListViewItem *, const QPoint & ) ) );
    // ### connect dragged(...), ...

    hlay->addWidget( mIdentityList, 1 );

    QVBoxLayout *vlay = new QVBoxLayout(); // inherits spacing
    hlay->addLayout( vlay );

    QPushButton *button = new QPushButton( i18n( "&Add..." ), this );
    mModifyButton = new QPushButton( i18n( "&Modify..." ), this );
    mRenameButton = new QPushButton( i18n( "&Rename" ), this );
    mRemoveButton = new QPushButton( i18n( "Remo&ve" ), this );
    mSetAsDefaultButton = new QPushButton( i18n( "Set as &Default" ), this );
    button->setAutoDefault( false );
    mModifyButton->setAutoDefault( false );
    mModifyButton->setEnabled( false );
    mRenameButton->setAutoDefault( false );
    mRenameButton->setEnabled( false );
    mRemoveButton->setAutoDefault( false );
    mRemoveButton->setEnabled( false );
    mSetAsDefaultButton->setAutoDefault( false );
    mSetAsDefaultButton->setEnabled( false );
    connect( button, SIGNAL( clicked() ),
             this, SLOT( slotNewIdentity() ) );
    connect( mModifyButton, SIGNAL( clicked() ),
             this, SLOT( slotModifyIdentity() ) );
    connect( mRenameButton, SIGNAL( clicked() ),
             this, SLOT( slotRenameIdentity() ) );
    connect( mRemoveButton, SIGNAL( clicked() ),
             this, SLOT( slotRemoveIdentity() ) );
    connect( mSetAsDefaultButton, SIGNAL( clicked() ),
             this, SLOT( slotSetAsDefault() ) );
    vlay->addWidget( button );
    vlay->addWidget( mModifyButton );
    vlay->addWidget( mRenameButton );
    vlay->addWidget( mRemoveButton );
    vlay->addWidget( mSetAsDefaultButton );
    vlay->addStretch( 1 );
    load();
}

KCMIdentity::~KCMIdentity()
{
}


void KCMIdentity::load()
{
    KPIMIdentities::IdentityManager *im = m_identityManager;
    mOldNumberOfIdentities = im->shadowIdentities().count();
    // Fill the list:
    mIdentityList->clear();
    QTreeWidgetItem *item = 0;
    for ( KPIMIdentities::IdentityManager::Iterator it = im->modifyBegin(); it != im->modifyEnd(); ++it ) {
        item = new IdentityListViewItem( mIdentityList, item, *it );
    }
    if ( mIdentityList->currentItem() ) {
        mIdentityList->currentItem()->setSelected( true );
    }
}

void KCMIdentity::save()
{
    Q_ASSERT( !mIdentityDialog );

    m_identityManager->sort();
    m_identityManager->commit();
}

void KCMIdentity::slotNewIdentity()
{
    Q_ASSERT( !mIdentityDialog );

    KPIMIdentities::IdentityManager *im = m_identityManager;
    NewIdentityDialog dialog( im->shadowIdentities(), this );
    dialog.setObjectName( "new" );

    if( dialog.exec() == QDialog::Accepted ) {
        QString identityName = dialog.identityName().trimmed();
        Q_ASSERT( !identityName.isEmpty() );

        //
        // Construct a new Identity:
        //
        switch ( dialog.duplicateMode() ) {
        case NewIdentityDialog::ExistingEntry:
        {
            KPIMIdentities::Identity &dupThis = im->modifyIdentityForName( dialog.duplicateIdentity() );
            im->newFromExisting( dupThis, identityName );
            break;
        }
        case NewIdentityDialog::ControlCenter:
            im->newFromControlCenter( identityName );
            break;
        case NewIdentityDialog::Empty:
            im->newFromScratch( identityName );
        default: ;
        }

        //
        // Insert into listview:
        //
        KPIMIdentities::Identity &newIdent = im->modifyIdentityForName( identityName );
        QTreeWidgetItem *item = 0;
        if ( mIdentityList->selectedItems().size() > 0 ) {
            item = mIdentityList->selectedItems()[0];
        }

        QTreeWidgetItem * newItem = 0;
        if ( item ) {
            newItem = new IdentityListViewItem( mIdentityList, mIdentityList->itemAbove( item ), newIdent );
        } else {
            newItem = new IdentityListViewItem( mIdentityList, newIdent );
        }

        mIdentityList->selectionModel()->clearSelection();
        if ( newItem ) {
            newItem->setSelected( true );
        }

        slotModifyIdentity();
    }
}

void KCMIdentity::slotModifyIdentity()
{
    Q_ASSERT( !mIdentityDialog );

    IdentityListViewItem *item = 0;
    if ( mIdentityList->selectedItems().size() > 0 ) {
        item = dynamic_cast<IdentityListViewItem*>( mIdentityList->selectedItems()[0] );
    }
    if ( !item ) {
        return;
    }

    mIdentityDialog = new IdentityDialog( this );
    mIdentityDialog->setIdentity( item->identity() );

    // Hmm, an unmodal dialog would be nicer, but a modal one is easier ;-)
    if ( mIdentityDialog->exec() == QDialog::Accepted ) {
        mIdentityDialog->updateIdentity( item->identity() );
        item->redisplay();
        emit changed( true );
    }

    delete mIdentityDialog;
    mIdentityDialog = 0;
}

void KCMIdentity::slotRemoveIdentity()
{
    Q_ASSERT( !mIdentityDialog );

    KPIMIdentities::IdentityManager *im = m_identityManager;
    kFatal( im->shadowIdentities().count() < 2, 5006 )
        << "Attempted to remove the last identity!";

    IdentityListViewItem *item = 0;
    if ( mIdentityList->selectedItems().size() > 0 ) {
        item = dynamic_cast<IdentityListViewItem*>( mIdentityList->selectedItems()[0] );
    }
    if ( !item ) {
        return;
    }

    QString msg = i18n( "<qt>Do you really want to remove the identity named "
                        "<b>%1</b>?</qt>", item->identity().identityName() );
    if( KMessageBox::warningContinueCancel( this, msg, i18n("Remove Identity"),
                                            KGuiItem(i18n("&Remove"),
                                                     "edit-delete") )
        == KMessageBox::Continue ) {
        if ( im->removeIdentity( item->identity().identityName() ) ) {
            delete item;
            if ( mIdentityList->currentItem() ) {
                mIdentityList->currentItem()->setSelected( true );
            }
            refreshList();
        }
    }
}

void KCMIdentity::slotRenameIdentity()
{
    Q_ASSERT( !mIdentityDialog );

    QTreeWidgetItem *item = 0;

    if ( mIdentityList->selectedItems().size() > 0 ) {
        item = mIdentityList->selectedItems()[0];
    }
    if ( !item ) return;

    mIdentityList->editItem( item );
}

void KCMIdentity::slotRenameIdentity( KMail::IdentityListViewItem *item , const QString &text )
{
    if ( !item ) return;

    QString newName = text.trimmed();
    if ( !newName.isEmpty() &&
         !m_identityManager->shadowIdentities().contains( newName ) ) {
        KPIMIdentities::Identity &ident = item->identity();
        ident.setIdentityName( newName );
        emit changed( true );
    }
    item->redisplay();
}

void KCMIdentity::slotContextMenu( KMail::IdentityListViewItem *item, const QPoint &pos )
{
    QMenu *menu = new QMenu( this );
    menu->addAction( i18n( "Add..." ), this, SLOT( slotNewIdentity() ) );
    if ( item ) {
        menu->addAction( i18n( "Modify..." ), this, SLOT( slotModifyIdentity() ) );
        if ( mIdentityList->topLevelItemCount() > 1 ) {
            menu->addAction( i18n( "Remove" ), this, SLOT( slotRemoveIdentity() ) );
        }
        if ( !item->identity().isDefault() ) {
            menu->addAction( i18n( "Set as Default" ), this, SLOT( slotSetAsDefault() ) );
        }
    }
    menu->exec( pos );
    delete menu;
}


void KCMIdentity::slotSetAsDefault()
{
    Q_ASSERT( !mIdentityDialog );

    IdentityListViewItem *item = 0;
    if ( mIdentityList->selectedItems().size() > 0 ) {
        item = dynamic_cast<IdentityListViewItem*>( mIdentityList->selectedItems()[0] );
    }
    if ( !item ) {
        return;
    }

    KPIMIdentities::IdentityManager *im = m_identityManager;
    im->setAsDefault( item->identity().uoid() );
    refreshList();
}

void KCMIdentity::refreshList()
{
    for ( int i = 0; i < mIdentityList->topLevelItemCount(); ++i ) {
        IdentityListViewItem *item = dynamic_cast<IdentityListViewItem*>( mIdentityList->topLevelItem( i ) );
        if ( item ) {
            item->redisplay();
        }
    }
    emit changed( true );
}

void KCMIdentity::slotIdentitySelectionChanged()
{
    IdentityListViewItem *item = 0;
    if ( mIdentityList->selectedItems().size() >  0 ) {
        item = dynamic_cast<IdentityListViewItem*>( mIdentityList->selectedItems()[0] );
    }

    mRemoveButton->setEnabled( item && mIdentityList->topLevelItemCount() > 1 );
    mModifyButton->setEnabled( item );
    mRenameButton->setEnabled( item );
    mSetAsDefaultButton->setEnabled( item && !item->identity().isDefault() );
}

#include "identitykcm.moc"
