
/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "sendmailelement.h"
#include "foaf.h"
#include "plug.h"
#include "socket.h"

#include <Soprano/QueryResultIterator>
#include <Soprano/Model>
#include <Soprano/Node>
#include <Soprano/Vocabulary/RDF>
#include <Soprano/Vocabulary/Xesam>
#include <Soprano/StatementIterator>

#include <mailtransport/transportmanager.h>
#include <mailtransport/transportjob.h>
#include <kpimidentities/identity.h>
#include <kpimidentities/identitymanager.h>
#include <kmime/kmime_message.h>

#include <plasma/widgets/label.h>
#include <plasma/widgets/pushbutton.h>

#include <KServiceTypeTrader>
#include <KCMultiDialog>
#include <KMessageBox>
#include <KMimeType>
#include <KCodecs>
#include <KIO/NetAccess>

#include <QtGui/QPainter>
#include <QtCore/QRegExp>
#include <QtCore/QFile>
#include <QtGui/QGraphicsGridLayout>
#include <KPushButton>

namespace {
    KMime::Message* createMessage( const KPIMIdentities::Identity& identity,
                                   const QString& toName,
                                   const QString& toEmail,
                                   const QString& subject,
                                   const QString& text,
                                   const QList<KUrl>& attachments )
    {
        QString name = identity.fullName();
        QString email = identity.emailAddr();

        Q_ASSERT( !name.isEmpty() );
        Q_ASSERT( !email.isEmpty() );

        // create the message
        // ==================
        KMime::Message* message = new KMime::Message();
        if ( attachments.count() ) {
            message->contentType()->setMimeType( "multipart/mixed" );
            message->contentType()->setBoundary( KMime::multiPartBoundary() );
        }
        else {
            message->contentType()->setMimeType( "text/plain" );
        }

        // add the sender
        // ==================
        KMime::Headers::From* fromAddress = new KMime::Headers::From( message );
        fromAddress->addAddress( email.trimmed().toLatin1(), name.toLatin1() );
        message->setHeader( fromAddress );

        // add the addressee
        // ==================
        KMime::Headers::To* toAddress = new KMime::Headers::To( message );
        toAddress->addAddress( toEmail.trimmed().toLatin1(), toName.toLatin1() );
//        KMime::Types::Mailbox Address;
//        Address.setAddress( QString( "%1 <%2>" ).arg( toName ).arg( toEmail ).trimmed().toLatin1() );
//        toAddress->addAddress( Address );
        message->setHeader( toAddress );

        // add the subject
        // ==================
        if ( !subject.isEmpty() ) {
            KMime::Headers::Subject* subjectHeader = new KMime::Headers::Subject;
            subjectHeader->fromUnicodeString( subject, "utf-8" );
            message->setHeader( subjectHeader );
        }

        // add the date
        // ==================
        KMime::Headers::Date* date = new KMime::Headers::Date;
        date->setDateTime( KDateTime::currentLocalDateTime() );
        message->setHeader( date );

        // TODO: Do we need a user agent?

        // add the message ID
        // ==================
        KMime::Headers::MessageID* mi = new KMime::Headers::MessageID;
        mi->generate( email.mid( email.indexOf( "@" )+1 ).toLatin1() );
        message->setHeader( mi );

        // add the actual content
        // ======================
        if ( attachments.count() ) {
            KMime::Content* content = new KMime::Content();
            content->contentType()->from7BitString( "text/plain" );
            content->fromUnicodeString( text );
            content->assemble();
            message->addContent( content );
        }
        else {
            message->fromUnicodeString( text );
        }

        // add the attachments
        // ===================
        for ( int i = 0; i < attachments.count(); ++i ) {
            KUrl url = attachments[i];
            QString tmpFileName;
            if ( !KIO::NetAccess::download( url, tmpFileName, 0 ) ) {
                // FIXME: inform user
                kDebug() << "Failed to retrieve" << url << KIO::buildErrorString( KIO::NetAccess::lastError(), KIO::NetAccess::lastErrorString() );
            }
            else {
                QFile file( tmpFileName );
                if ( file.open( QIODevice::ReadOnly ) ) {
                    QByteArray data = file.readAll();
                    file.close();

                    // convert to base64
                    QByteArray base64Data = KCodecs::base64Encode( data, true );

                    KMime::Content* c = new KMime::Content();
                    c->setBody( base64Data + "\n\n" );

                    // Add a content type / find mime-type
                    KMimeType::Ptr type = KMimeType::findByUrl( url );
                    KMime::Headers::ContentType* contentType = new KMime::Headers::ContentType( c );
                    contentType->fromUnicodeString( type->name(), "utf-8" ); // FIXME: this is weird, why do we need an encoding in a method called fromUnicodeString?
                    contentType->setName( url.fileName(), QByteArray() );
                    c->setHeader( contentType );

                    // Set the encoding.
                    KMime::Headers::ContentTransferEncoding* cte = new KMime::Headers::ContentTransferEncoding( c );
                    cte->setEncoding( KMime::Headers::CEbase64 );
                    cte->setDecoded( false );
                    c->setHeader( cte );
                    c->assemble();

                    message->addContent( c );
                }
                else {
                    // FIXME: inform user
                    kDebug() << "Failed to open" << tmpFileName;
                }

                KIO::NetAccess::removeTempFile( tmpFileName );
            }
        }

        message->assemble();
        return message;
    }
}


Konduit::Elements::SendMailElement::SendMailElement( QObject* parent, const QVariantList& args )
    : MemoryModelElement( parent, args )
{
    setHasConfigurationInterface( true );

    addPlug( "addresses" );
    addPlug( "content" );
    addPlug( "attachments" );
    addPlug( "subject" );

    m_sendMailButton = new Plasma::PushButton( this );
    m_sendMailButton->setText( "Send emails" );

    QGraphicsGridLayout* layout = new QGraphicsGridLayout( this );
    layout->addItem( m_sendMailButton, 1, 0 );
    layout->setRowFixedHeight( 0, 70 );

    connect( m_sendMailButton, SIGNAL( clicked() ), this, SLOT( slotSendMails() ) );

    setToolTip("Send Mail");
    resize( contentsSizeHint( Qt::PreferredSize, QSizeF() ) );
    setMinimumSize( QSizeF(95, 130) );
}


Konduit::Elements::SendMailElement::~SendMailElement()
{
}


QString Konduit::Elements::SendMailElement::extractEmailContent()
{
    Soprano::StatementIterator it = plug( "content" )->socket()->data();
    if ( it.next() ) {
        return it.current().object().toString();
    }
    else {
        return QString();
    }
}

QString Konduit::Elements::SendMailElement::extractEmailSubject()
{
    Soprano::StatementIterator it = plug( "subject" )->socket()->data();
    if ( it.next() ) {
        return it.current().object().toString();
    }
    else {
        return QString();
    }
}

QList<KUrl> Konduit::Elements::SendMailElement::extractAttachements()
{
    QList<KUrl> urls;

    // We prefer the clean way
    // =======================
    if( Soprano::Model* m = model( plug("attachments") ) ) {
        Soprano::QueryResultIterator it = m->executeQuery( QString("prefix xesam: <%1> "
                                                                   "select distinct ?url where { ?r a xesam:File . "
                                                                   "?r xesam:url ?url . }")
                                                           .arg( Soprano::Vocabulary::Xesam::xesamNamespace().toString() ),
                                                           Soprano::Query::QueryLanguageSparql );
        while( it.next() ) {
            urls << it.binding("url").toString();
        }
        delete m;
    }

    // ...but fallback in case we cannot create a memory model
    // (and to show the different ways data can be handled)
    // =======================================================
    else {
        Soprano::StatementIterator it = plug( "attachments" )->data();
        while ( it.next() ) {
            Soprano::Statement s = *it;
            if ( s.predicate() == Soprano::Vocabulary::Xesam::url() ) {
                urls << s.object().toString();
            }
        }
    }
    return urls;
}


QString Konduit::Elements::SendMailElement::formatEmailContent( Soprano::Model* model, const QString& text, const QUrl& resource )
{
    static QRegExp rdfTagRx( "<([^\\s]+)>" );

    QString newText( text );
    int i = 0;
    while( ( i = newText.indexOf( rdfTagRx, i ) ) >= 0 ) {

        // extract the tag we like to replace
        QString tag = rdfTagRx.cap( 1 );

        // expand foaf namespace the ugly way
        // FIXME make this nicer
        if ( tag.startsWith( "foaf:" ) ) {
            tag.replace( "foaf:", Soprano::Vocabulary::FOAF::foafNamespace().toString() );
        }

        // query the data
        Soprano::QueryResultIterator it = model->executeQuery( QString( "select ?x where { <%1> <%2> ?x . }" )
                                                               .arg( resource.toString() )
                                                               .arg( tag ),
                                                               Soprano::Query::QueryLanguageSparql );
        if ( it.next() ) {
            newText.replace( i, rdfTagRx.matchedLength(), it.binding( "x" ).toString() );
            it.close();
        }
        else {
            kDebug() << "Could not find replacement for" << tag;
            newText.replace( i, rdfTagRx.matchedLength(), "NULL" );
        }
    }

    return newText;
}


void Konduit::Elements::SendMailElement::slotSendMails()
{
    KPIMIdentities::IdentityManager identityManager( true );
    const KPIMIdentities::Identity& defaultIdentity = identityManager.defaultIdentity();

    // Allow the selection of an identity
    kDebug() << "Default identity:" << defaultIdentity.fullName() << defaultIdentity.emailAddr();

    QString transportName = defaultIdentity.transport();
    if ( transportName.isEmpty() ) {
        transportName = MailTransport::TransportManager::self()->defaultTransportName();
    }

    if ( transportName.isEmpty() ) {
        KMessageBox::sorry( 0, i18n( "Please configure a transport for email sending." ) );
        return;
    }

    if ( !plug( "content" )->socket() ) {
        KMessageBox::sorry( 0, i18n( "Please connect the content input." ) );
        return;
    }

    QString emailContent = extractEmailContent();
    QList<KUrl> attachments = extractAttachements();

    if ( Soprano::Model* m = model( plug( "addresses" ) ) ) {
        Soprano::QueryResultIterator it = m->executeQuery( QString( "select ?r ?name ?email where { "
                                                                    "?r <%1> <%2> . "
                                                                    "?r <%3> ?email . "
                                                                    "?r <%4> ?name . }" )
                                                           .arg( Soprano::Vocabulary::RDF::type().toString() )
                                                           .arg( Soprano::Vocabulary::FOAF::Person().toString() )
                                                           .arg( Soprano::Vocabulary::FOAF::mbox().toString() )
                                                           .arg( Soprano::Vocabulary::FOAF::name().toString() ),
                                                           Soprano::Query::QueryLanguageSparql );

        while ( it.next() ) {
            QString name = it.binding( "name" ).toString();
            QString email = it.binding( "email" ).uri().path();
            QString text = formatEmailContent( m, emailContent, it.binding( "r" ).uri() );
            QString subject = extractEmailSubject();

            kDebug() << "Writing email to" << name << "(" << email << ")";

            if ( KMessageBox::questionYesNo( 0,
                                             i18n( "This is what I will send:\n"
                                                   "From: %1\n"
                                                   "To:   %2\n"
                                                   "Text:\n"
                                                   "%3\n"
                                                   "OK?",
                                                   defaultIdentity.fullEmailAddr(),
                                                   email,
                                                   text ) ) == KMessageBox::Yes ) {
                MailTransport::TransportJob* transport = MailTransport::TransportManager::self()->createTransportJob( transportName );

                KMime::Message* message = createMessage( defaultIdentity, name, email, subject, text, attachments );

                transport->setSender( defaultIdentity.emailAddr() );
                transport->setTo( QStringList() << email );
                transport->setData( message->encodedContent( true ) + "\n\r" );

                // KJobs are "fire and forget", i.e. they will delete themselves
                MailTransport::TransportManager::self()->schedule( transport );
            }
        }
    }
}


void Konduit::Elements::SendMailElement::showKonduitConfigurationInterface()
{
    KCMultiDialog dlg( 0 );
    dlg.setButtons( KDialog::Ok|KDialog::Apply|KDialog::Cancel|KDialog::Default );
    dlg.setDefaultButton( KDialog::Ok );
    dlg.addModule( "kcm_identity" );
    dlg.addModule( "kcm_mailtransport" );
    dlg.exec();
}



QSizeF Konduit::Elements::SendMailElement::contentsSizeHint( Qt::SizeHint which, const QSizeF& constraint ) const
{
    // do NOT call Element::sizeHint here. That will only result in an endless loop
    QSizeF size = Applet::sizeHint( which, constraint );
    size.setWidth( qMax( size.width(), (qreal) m_sendMailButton->nativeWidget()->sizeHint().width() + 70 ) );
    return size;
}


void Konduit::Elements::SendMailElement::paintInterface( QPainter* painter,
                                                         const QStyleOptionGraphicsItem* option,
                                                         const QRect& contentsRect )
{
    Q_UNUSED( option );
    painter->save();
    painter->translate( contentsRect.left(), contentsRect.top() );

    // we draw some small lines to indicate the duplexing
    painter->setPen( QPen( Qt::white, 2 ) );
    painter->drawLine( 0, 10, 20, 10 );
    painter->drawLine( 0, 40, 20, 40 );
    painter->drawLine( 0, 70, 20, 70 );

    KIcon contentIcon( "document-import" );
    KIcon addressIcon( "contact-new" );
    KIcon attachmentIcon( "document-open-folder" );

    painter->drawPixmap( 22, 2, 16, 16, addressIcon.pixmap( QSize( 16, 16 ) ) );
    painter->drawPixmap( 22, 32, 16, 16, contentIcon.pixmap( QSize( 16, 16 ) ) );
    painter->drawPixmap( 22, 62, 16, 16, attachmentIcon.pixmap( QSize( 16, 16 ) ) );

    painter->restore();
}

#include "sendmailelement.moc"
