/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "socket.h"
#include "source.h"
#include "plug.h"

#include <Soprano/StatementIterator>

#include <QtGui/QPainter>
#include <QtCore/QPointer>

#include <KDebug>


class Konduit::Socket::Private
{
public:
    Private()
        : parentSource( 0 ),
          source( 0 ),
          boundingRect( 0, 0, 20, 20 ) {
    }

    Source* parentSource;
    QPointer<Plug> source;
    QString name;

    QRectF boundingRect;
};


Konduit::Socket::Socket( Source* source, const QString& name )
    : QObject( 0 ),
      QGraphicsItem( dynamic_cast<QGraphicsItem*>( source )->parentItem() ),
      d( new Private() )
{
    d->parentSource = source;
    d->name = name;
    setZValue( dynamic_cast<QGraphicsItem*>( source )->zValue()+1 );

    setAcceptDrops( true );

    setToolTip( name );
}


Konduit::Socket::~Socket()
{
    unplug();
    delete d;
}


QString Konduit::Socket::name() const
{
    return d->name;
}


Konduit::Source* Konduit::Socket::source() const
{
    return d->parentSource;
}


bool Konduit::Socket::plugIn( Plug* source )
{
    if ( d->source != source ) {
        Plug* oldPlug = d->source;
        unplug();
        d->source = source;
        update();
        d->source->plugIn( this );

        emit plugStateChanged( oldPlug, source );
    }
    return true;
}


void Konduit::Socket::unplug()
{
    if ( d->source ) {
        Plug* p = d->source;
        d->source = 0;
        p->unplug();

        emit plugStateChanged( p, 0 );
    }
}


Konduit::Plug* Konduit::Socket::plug() const
{
    return d->source;
}


Soprano::StatementIterator Konduit::Socket::data() const
{
    return d->parentSource->data( this );
}


QRectF Konduit::Socket::boundingRect() const
{
    return d->boundingRect;
}


void Konduit::Socket::paint( QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget )
{
    painter->fillRect( 0, 0, 20, 20, Qt::white );
    painter->drawEllipse( 2, 2, 16, 16 );
    painter->fillRect( 5, 9, 2, 2, Qt::black );
    painter->fillRect( 13, 9, 2, 2, Qt::black );
}


QVariant Konduit::Socket::itemChange( GraphicsItemChange change, const QVariant& value )
{
    if ( change == QGraphicsItem::ItemPositionHasChanged ) {
        if ( plug() ) {
            plug()->updatePosition();
        }
    }

    return QGraphicsItem::itemChange( change, value );
}

#include "socket.moc"
