/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "source.h"
#include "socket.h"
#include "element.h"

#include <QtCore/QMap>
#include <QtCore/QPointer>


class Konduit::Source::Private
{
public:
    // We use QPointer since the sockets may have already been deleted
    // by the QGraphicsScene
    // we cannot use QMap since we want the sockets to be sorted by insertion time
    QList<QPointer<Socket> > sockets;

    Socket* findSocket( const QString& name ) {
        for ( int i = 0; i < sockets.count(); ++i ) {
            if ( sockets[i]->name() == name ) {
                return sockets[i];
            }
        }
        return 0;
    }
};


Konduit::Source::Source()
    : d( new Private() )
{
}


Konduit::Source::~Source()
{
    qDeleteAll( d->sockets );
    delete d;
}


int Konduit::Source::countSockets() const
{
    return d->sockets.count();
}


QStringList Konduit::Source::allSocketNames() const
{
    QStringList names;
    foreach( QPointer<Socket> socket, d->sockets ) {
        names << socket->name();
    }
    return names;
}


QList<Konduit::Socket*> Konduit::Source::allSockets() const
{
    QList<Konduit::Socket*> l;
    Q_FOREACH( QPointer<Socket> p, d->sockets ) {
        l.append( p );
    }
    return l;
}


Konduit::Socket* Konduit::Source::socket( const QString& name ) const
{
    if ( !name.isEmpty() ) {
        return d->findSocket( name );
    }
    else if ( countSockets() == 1 ) {
        return d->sockets.first();
    }
    return 0;
}


Konduit::Socket* Konduit::Source::addSocket( const QString& name )
{
    Q_ASSERT( !d->findSocket( name ) );

    // FIXME: possible memleak
    Socket* p = new Socket( this, name );
    d->sockets.append( p );
    p->connect( p,
                SIGNAL( plugStateChanged( Konduit::Plug*, Konduit::Plug* ) ),
                dynamic_cast<Element*>( this ),
                SLOT( slotPlugStateChanged( Konduit::Plug*, Konduit::Plug* ) ) );
    return p;
}


Konduit::Socket* Konduit::Source::createSocket( Source* source, const QString& name )
{
    Socket* p = new Socket( source, name );
    return p;
}
