/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef VISUALQUERYBUILDER_H
#define VISUALQUERYBUILDER_H

#include "konduit_export.h"

#include "combobox.h"
#include "querythread.h"

#include <qvariant.h>
#include <QListWidget>
#include <QListWidgetItem>
#include <Qt3Support/Q3MimeSourceFactory>
#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QLabel>
#include <QtGui/QLineEdit>
#include <QtGui/QPushButton>
#include <QtGui/QGroupBox>
#include <QtGui/QTextEdit>
#include <QtGui/QComboBox>
#include <QCompleter>
#include <QThread>

#include <Soprano/Soprano>
#include <Soprano/Model>
#include <Soprano/Client/DBusClient>
#include <Soprano/Client/DBusModel>
#include <Soprano/Util/DummyModel>

namespace Konduit {

/*!
    Visual interface for building SPARQL queries
    Everything suffixed with C relates to the Conditions group and
    Everything suffixed with O relates to the Outputs group  
*/

class KONDUIT_EXPORT VisualQueryBuilder : public QWidget
{
    Q_OBJECT

public:
    VisualQueryBuilder(QWidget* parent = 0, Qt::WindowFlags fl = 0, QString initialQuery = QString());
    ~VisualQueryBuilder();

public Q_SLOTS:
    void buttonOK_clicked();
    void buttonApply_clicked();
    void buttonCancel_clicked();
    void buttonAddO_clicked();
    void buttonAddC_clicked();
    void buttonRemoveO_clicked();
    void buttonRemoveC_clicked();
    void insertValueC( QString value );
    void insertValueO( QString value );
    void setCompletersCS( QStringList compList );//sets the completers from the autocompletion list (of the Subject field of the Conditions part)
    void setCompletersCP( QStringList compList );//sets the completers from the autocompletion list (of the Predicate field of the Conditions part)
    void setCompletersCO( QStringList compList );//sets the completers from the autocompletion list (of the Object field of the Conditions part)
    void setCompletersOS( QStringList compList );//sets the completers from the autocompletion list (of the Subject field of the Output part)
    void setCompletersOP( QStringList compList );//sets the completers from the autocompletion list (of the Predicate field of the Output part)
    void setCompletersOO( QStringList compList );//sets the completers from the autocompletion list (of the Object field of the Output part)
    /** Initializes the GUI based on the initial query
    */
    void processInitialQuery( QString initialQuery );

Q_SIGNALS:
    void queryChanged( QString query );

private Q_SLOTS:
    void languageChange();
    void textChangedCO();//repopulates types list
    void textChangedOO();//repopulates types list
    void refreshTypes(); // refresh type comboboxes of the objects (from changing list)
    void editingFinishedO();
    void editingFinishedC();
    void varsChanged();//repopulates variable list


private:
    //FIXME: put every field into the Private class
    QPushButton *buttonOK;
    QPushButton *buttonCancel;
    QPushButton *buttonApply;
    QGroupBox *groupBoxC;
    QLabel *textLabel1_2_4;
    QLabel *textLabel1_2_2_3;
    QLabel *textLabel1_4;
    QLineEdit *lineEditCS;
    ComboBox *comboBoxCP;
    QLineEdit *lineEditCO;
    QPushButton *pushButtonAddC;
    QPushButton *pushButtonRemoveC;
    QTextEdit *textEditC;
    QGroupBox *groupBoxO;
    QLabel *textLabel1_2;
    QLabel *textLabel1_2_2;
    QLabel *textLabel1;
    ComboBox *comboBoxOP;
    QLineEdit *lineEditOO;
    QLineEdit *lineEditOS;
    QPushButton *pushButtonAddO;
    QPushButton *pushButtonRemoveO;
    QTextEdit *textEditO;
    QListWidget *listWidgetO;
    QListWidget *listWidgetC;
    QComboBox *comboBoxTypesC;
    QComboBox *comboBoxTypesO;

    //QString query;

    /** Initializes the predicates, the prefixes and the types
    */
    void init();
    /** Initializes the UI components
    */
    void setupUi();
    /** Initializes the names in the UI  using the current language.
    */
    void retranslateUi();

    /** Returns the query based on the the listwidgets' contents
    */
    QString constructQuery();
    
    /** Sets the background color of the three lineedit objects
    */
    void setColor( QLineEdit *lineEditS, QLineEdit *lineEditP, QLineEdit *lineEditO, QString objectType, QColor color );
    
    /** Returns the list of all the variables used (already added to the listWidgets)
    */
    QStringList getVars();

    //FIXME: put every field into the Private class
    QList<QString> predicates;
    QList<QString> typesC;//types for Conditional (WHERE) expressions
    QList<QString> typesO;//types for Output (CONSTRUCT) expressions
    QList<QString> allTypes;//unchaging list holding all possible types
    QMap<QString, QString> prefixes;
    QCompleter* cCompleter;
    QList<QString> autoCompletionListCS;//autocomplete list for lineEditCS
    QueryThread* qTh;//used for non-thread querying
    QStringList varList;
    QString lastQueryO;//holds the last autocompletion query of the Output part
    QString lastQueryC;//holds the last autocompletion query of the Conditional part
    QString lastObjectO;//holds the last object text of the Output part
    QString lastObjectC;//holds the last object text of the Conditional part

};

}

#endif // VISUALQUERYBUILDER_H
